
var balloonhelp = {

	// Set this to false to disable debug output
	BALLOON_DEBUG : false,
	BALLOON_NORMAL_TIMEOUT  : 2000,				// 2 seconds for normal balloons
	BALLOON_TOOLTIP_TIMEOUT : 2000,				// 2 seconds for tooltips
	BALLOON_FEATURE_TIMEOUT : 8000,				// 8 seconds for extended balloon help

	BALLOON_DELAY_NORMAL_TIMEOUT  : 0,		// No delay for normal balloons
	BALLOON_DELAY_TOOLTIP_TIMEOUT : 500,	// 1/2 a second for tooltips
	BALLOON_DELAY_FEATURE_TIMEOUT : 0,		// No delay for extended balloon help
	BALLOON_FULLY_VISIBLE_DEADLINE: 1000, // The delay before the balloon is fully visible, in case the transparency misses some cycles

	TRANSPARENCY_INC : 30,								// Amount to (inc/dec)rement the transparency of the balloon when fading in/out
	TRANSPARENCY_MIN : 0,									// Min amount to decrease the transparency
	TRANSPARENCY_MAX : 255,								// Max amount to increase the transparency
	TRANSPARENCY_TIMEOUT : 5,							// Time to wait before incrementing the transparency to fade in/out
	TRANSPARENCY_DELAY : 0,								// Time to wait before starting the fade

	mBaloonHelpTimer: null,								// This is the timer to hide the balloon (default 2 seconds for normal, 8 for extended)
	mBalloonHelpShowTimer: null,					// This is the timer to show a delayed balloon (simular to tooltips)
	mBalloonFadeTimer: null,
	mBalloonHelpID: 'BalloonHelpTip',			// ID of the popup (Held in the <popupset> tag)
	mBalloonHelpTimeout: 2000,						// Timeout before hiding balloon help
	mBalloonHelpDelayTimeout: 500,				// 1/2 second for the delay before the balloon help to appear
	mMouseYOffset: 0,
	mMouseXOffset: 0,
	mAnchorNode: null,

	mIsNested : false,

	strBundle: null,											// String bundle for .properties information

	// Held for delayed popup
	mElementID: null,											// Element we are to show up on
	mUrlID: null,													// Url we are poping up on so we can turn off individual balloons.
	mInTitle: null,												// Title to display
	mInToolTip: null,											// Tooltip to display if bubbles are turned off
	mInDescription: null,									// Description of balloon help
	mWidgetPref: null,										// Preference linked to this widget type
	mIsFeature: null,											// To show the extended balloon, or not to show.

	xPos: 0,															// XPosition of balloon or tooltip popup
	yPos: 0,															// YPosition of balloon or tooltip popup

	datFileDir : null,										// Data file directory (featureoptions.dat)
	FEATURE_OPTIONS_FILE : "featureoptions.dat",		// Feature options file (to turn off particular feature discovery balloons

	// File open flags (from prio.h)
	PR_RDONLY   : 1,
	PR_WRONLY   : 2,
	PR_TRUNCATE : 4,

	/*************************************************************************************************************************
	 * debug - This will display a message on the console if BALLOON_DEBUG is true
	 ************************************************************************************************************************/
	debug : function(msg) {
			if (this.BALLOON_DEBUG) {
				dump('balloonhelp.js: ' + msg + '\n');
			}
	},

	/*************************************************************************************************************************
	 * GetFeatureOptionsFile - This will locate the feature options file in the dat directory
	 * datFileDir needs to be initialized first
	 ************************************************************************************************************************/
	GetFeatureOptionsFile : function() {
		if (!this.datFileDir) return null;

		// Create file descriptor
		if (this.datFileDir) {
		 var file = this.datFileDir.clone();
		 file.append(this.FEATURE_OPTIONS_FILE);
		 return file; // returns nsILocalFile
		}
	   else
	    return null;
	},

	/*************************************************************************************************************************
	 * EnsureFeatureOptionsFileExists - This will create the feature options file if it does not exist
	 ************************************************************************************************************************/
	EnsureFeatureOptionsFileExists : function() {
		// Get the options file object
		var file = this.GetFeatureOptionsFile();
		if (!file)
			return null;

		// Check if the file exists, create it if it does not
		if (!file.exists()) {
			file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0);
		}
		return file; // returns nsILocalFile
	},

	/*************************************************************************************************************************
	 * ReadFeatureOptions - This will read in the entire feature options file to get all the feature discoverys that
   * are turned off
	 ************************************************************************************************************************/
	ReadFeatureOptions : function() {

		// Make sure that we have an options file
		var inFile = this.EnsureFeatureOptionsFileExists();

		if (inFile) {
			// Init the file input stream
			var fileInputStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
			                  .createInstance(Components.interfaces.nsIFileInputStream);
			fileInputStream.init(inFile, this.PR_RDONLY, 0, 0);

			// Init a scriptable input stream
			var fileInputScriptStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
			                  .createInstance(Components.interfaces.nsIScriptableInputStream);
			fileInputScriptStream.init(fileInputStream);

			// Read the file
			var fileContents = fileInputScriptStream.read(fileInputScriptStream.available());

			// Close file
			fileInputStream.close();

			// Return the lines as an array
			return fileContents.split('\n');
	   }
	   return null;
	},

	/*************************************************************************************************************************
	 * SaveFeatureOptions - This will save all the feature discovery balloons that we do not want to show
	 ************************************************************************************************************************/
	SaveFeatureOptions : function(newFeatureOptions) {
		// Make sure our file exists
		var outfile = this.EnsureFeatureOptionsFileExists();
		if (outfile) {
			var outFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
										.createInstance(Components.interfaces.nsIFileOutputStream);
			outFileStream.init(outfile, this.PR_WRONLY, 0, 0);

			for (var iLineCount = 0; iLineCount < newFeatureOptions.length; iLineCount++)
			{
				this.debug("Writing line: [" + iLineCount + "] " + newFeatureOptions[iLineCount]);
				outFileStream.write(newFeatureOptions[iLineCount] + '\n', newFeatureOptions[iLineCount].length + 1);
			}
			outFileStream.close();
		}
	},

	/*************************************************************************************************************************
	 * SetFeaturePref - This will turn on or off a particular feature discovery balloon tip
	 ************************************************************************************************************************/
	SetFeaturePref : function(nUri, turnOff) {
		// Read the options file first
		var optionsList = this.ReadFeatureOptions();
		var retFeaturePref = true;

		this.debug('Setting preference: ' + nUri + ' turning ' + (turnOff ? 'Off' : 'On'));
		if (optionsList) {
			nUri = nUri.toLowerCase();
			var bFoundMatch = false;
			var iLineCount;

			this.debug('Searching array of urls');
			for (iLineCount = 0; iLineCount < optionsList.length && !bFoundMatch; iLineCount++)
			{
				this.debug('Checking line: ' + iLineCount);
				if (optionsList.length == 0 || optionsList[iLineCount].indexOf('\t') == -1)
				{
					this.debug('Empty line removing');
					optionsList.splice(iLineCount,1);
				}	else {
					var featureInfo;
					if (optionsList[iLineCount].indexOf('\t') != -1)
						featureInfo = optionsList[iLineCount].split('\t');
					else
						featureInfo[0] = optionsList[iLineCount];
					this.debug('Comparing ' + featureInfo[0].toLowerCase());
					if (nUri == featureInfo[0].toLowerCase())
					{
						this.debug('Found Element Option for ' + nUri);
						if (!turnOff) {
							this.debug('Removing element ' + nUri);
							optionsList.splice(iLineCount,1);
							bFoundMatch = true;
						}
						retFeaturePref = false;	// We found the element so it is turned off
					}
				}
			}

			this.debug('Checking if we should add this one to the list ' + retFeaturePref + ' && ' + turnOff);
			if (retFeaturePref && turnOff) {
				this.debug('Adding new element ' + nUri);
				optionsList.push(nUri + '\t');
			}
		} else {
			if (turnOff)
			{
				// No elements in the list yet so add this first one
				this.debug('Adding first element ' + nUri);
				optionsList.push(nUri + '\t');
			}
		}

		// Now save the files
		this.SaveFeatureOptions(optionsList);
	},

	/*************************************************************************************************************************
	 * TurnOnAllFeatures - This will turn all feature discovery balloons back on (basically clears the file)
	 ************************************************************************************************************************/
	TurnOnAllFeatures : function() {
		var outfile = this.EnsureFeatureOptionsFileExists();
		if (outfile) {
			var outFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
										.createInstance(Components.interfaces.nsIFileOutputStream);
			outFileStream.init(outfile, this.PR_WRONLY, 0, 0);
			var outSeekFileStream = outFileStream.QueryInterface(Components.interfaces.nsISeekableStream);
			outSeekFileStream.setEOF();	// Truncate the file
			outFileStream.close();
		}
	},

	/*************************************************************************************************************************
	 * GetFeaturePref - This will determine if we should show a particular feature
	 ************************************************************************************************************************/
	GetFeaturePref : function(nUri) {
		var optionsList = this.ReadFeatureOptions();
		var retFeaturePref = true;

		this.debug('Looking if ' + nUri + ' should show a balloon');
		if (optionsList) {
			nUri = nUri.toLowerCase();
			var bFoundMatch = false;
			var iLineCount;

			this.debug('Searching array for ' + nUri);
			for (iLineCount = 0; iLineCount < optionsList.length && !bFoundMatch; iLineCount++)
			{
				var featureInfo;
				if (optionsList[iLineCount].indexOf('\t') != -1)
				{
					featureInfo = optionsList[iLineCount].split('\t');
					this.debug('Comparing ' + featureInfo[0].toLowerCase() + ' to ' + nUri);
					if (nUri == featureInfo[0].toLowerCase())
					{
						this.debug('Found Element Option for ' + nUri);
						bFoundMatch = true;
						retFeaturePref = false;	// We found the element so it is turned off
					}
				}
			}
		}

		return retFeaturePref
	},

	/*************************************************************************************************************************
	 * Init - Initialize the balloon help, sets up the feature options file
	 ************************************************************************************************************************/
	Init : function(nBalloonHelpID) {
		if (nBalloonHelpID)
			mBalloonHelpID = nBalloonHelpID;

		// Get the working directory based on DefRt
		// Data file stored in /dist/bin/defaults/featureoptions.dat (for the debug build)

		if (!this.datFileDir) {
			this.datFileDir = Components.classes["@mozilla.org/file/directory_service;1"]
			                            .getService(Components.interfaces.nsIProperties)
			                            .get("PrefD", Components.interfaces.nsILocalFile);
		}

		try {
			this.debug("Loading string bundle");
			this.strBundle = document.getElementById('bh_balloon');
			if (!this.strBundle)
				this.debug("Unable to load string bundle: unknown error");
		} catch (err) {
			this.debug("Unable to load string bundle: " + err);
		}

	},

	/*************************************************************************************************************************
	 * GetBalloonString - This will get a string from the stringbundle or uses a default
	 ************************************************************************************************************************/
	GetBalloonString : function(mStringName, mStringDefault)
	{
		var outString = null;
		if (mStringDefault)
			outString = mStringDefault;

		try {
			this.debug("Geting string for : " + mStringName);
			outString = this.strBundle.getString(mStringName);
		} catch (err) {
			this.debug("Unable to get string for : " + mStringName + ", Check that stringbundle 'bh_balloon' is available - " + err);
		}

		return outString;
	},

	/*************************************************************************************************************************
	 * toggleBalloonHelpPrefernece - This will turn on or off the general.enableWidgetDiscovery preference, true = on
	 * This will be linked to the small balloon in the top right corner of windows that allow balloon help.
	 ************************************************************************************************************************/
	ToggleBalloonFeaturePreference : function(forcePref)
	{
		this.debug('ToggleBalloonFeaturePreference('+forcePref+')');

		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
		                      .getService(Components.interfaces.nsIPrefBranch);
		var newValue = false;

		if (forcePref == null) {
			if (!prefs.getBoolPref("general.enableFeatureDiscovery"))
				newValue = true;
		} else if (forcePref == true || forcePref == false) {
			newValue = forcePref;
		} else {
			dump("in ToggleBalloonFeaturePreference - forcePref not a boolean: " + forcePref + "\n");
		}
		prefs.setBoolPref("general.enableFeatureDiscovery", newValue);
	},

	/*************************************************************************************************************************
	 * toggleBalloonHelpPreference - This will turn on or off the general.enableBalloonHelp preference, true = on
	 * This will be linked to the checkbox on the extended section of the balloon help.
	 ************************************************************************************************************************/
	ToggleBalloonHelpPreference : function()
	{
		this.debug('ToggleBalloonHelpPreference()');

		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
		                      .getService(Components.interfaces.nsIPrefBranch);
		var newValue = false;
		if (!prefs.getBoolPref("general.enableBalloonHelp"))
			newValue = true;

		prefs.setBoolPref("general.enableBalloonHelp", newValue);

		// Update the Menu check if we are in the browser
		var bubbleMenu = document.getElementById("balloonHelpMenu");
		if (!bubbleMenu) {
			// This will get the bubbleMenu from the pref-advanced dialog
			if (self.parent && parent.opener && parent.opener.document)
			{
				bubbleMenu = parent.opener.document.getElementById("balloonHelpMenu");
			}
		} else {
			// MERC (rpaul): if toggling is done from the dashboard window, the handle is to window.opener
			if (window.opener) {
				bubbleMenu = window.opener.document.getElementById('balloonHelpMenu');
			}		
		}
		this.debug("BubbleMenu: " + bubbleMenu);
		if (bubbleMenu) {
			if (newValue)
				bubbleMenu.setAttribute("checked", true);
			else
				bubbleMenu.removeAttribute("checked");
			this.debug("BubbleMenu: " + bubbleMenu.getAttribute("checked"));
		}
		
		// Update the broadcaster
		var balloonBroadcaster = document.getElementById('balloonBroadcaster');
		if (!balloonBroadcaster) {
			balloonBroadcaster = window.opener.document.getElementById('balloonBroadcaster');
			this.debug('balloonBroadcaster got from parent');
		}
		
		if (balloonBroadcaster) {
			balloonBroadcaster.setAttribute('showballoon', (newValue ? "true" : "false"));
		} 
		// update the dashboard balloon toggle button if its open
		if (dashboard.mDashboardWindow != null && dashboard.mDashboardWindowLoaded) {
			dashboard.mDashboardWindow.ToggleDashboardBalloonHelp(newValue);
		}
	},

	/*************************************************************************************************************************
	 * getBalloonWidgetPreference() - The will return true or false depending on the prefrences for the balloon help.
	 ************************************************************************************************************************/
	GetBalloonFeaturePreference : function()
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		var prefReturn = false;

		try {
	  		if (prefs.getBoolPref("general.enableFeatureDiscovery")) {
		    	prefReturn = prefs.getBoolPref("general.enableFeatureDiscovery");
			}
		} catch(err) {
   			this.debug("Unable to get balloon feature help preferences.");
   		}
		return prefReturn;
	},


	/*************************************************************************************************************************
	 * getBalloonHelpPreference() - The will return true or false depending on the prefrences for the balloon help.
	 ************************************************************************************************************************/
	GetBalloonHelpPreference : function()
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
		var prefReturn = false;

		try {
	  		if (prefs.getBoolPref("general.enableBalloonHelp")) {
		    	prefReturn = prefs.getBoolPref("general.enableBalloonHelp");
			}
		} catch(err) {
   			this.debug("Unable to get balloon help preferences.");
   		}
		return prefReturn;
	},

	/*************************************************************************************************************************
	 * checkBalloonHelpPrefs() - The will return true or false depending on the prefrences for the selected balloon
	 * help, isSearch will indicate if we are looking for the search or widget discovery.
	 ************************************************************************************************************************/
	CheckBalloonHelpPrefs : function()
	{
			var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
			var prefReturn = false;

			if (prefs)
			{
				switch(this.mWidgetPref)
				{
					case 'balloon':
					{
				  		if (prefs.getPrefType("general.enableBalloonHelp")) {
					    	prefReturn = prefs.getBoolPref("general.enableBalloonHelp");
						}
						return prefReturn;
					}break;
					case 'widget':
					case 'searchbar':
					default:
					{
					  if (prefs.getPrefType("general.enableFeatureDiscovery") == prefs.PREF_BOOL)
					  {
					    prefReturn = prefs.getBoolPref("general.enableFeatureDiscovery");
					  }
					}break;
				}
			}

			if (prefReturn)
			{
				if (!this.GetFeaturePref(this.mUrlID))
					return;
			}

			return prefReturn;
	},

	/*************************************************************************************************************************
	 * CancelBalloonHelpShow(event) - Cancels the timer to show the balloon help, see ShowBalloonHelpDelayed
	 * event is not used.
	 ************************************************************************************************************************/
	CancelBalloonHelpShow : function(event)
	{
		balloonhelp.debug('Cancelling the showing of balloon help.');
		if (balloonhelp.mBalloonHelpShowTimer) {
			clearTimeout(balloonhelp.mBalloonHelpShowTimer);
		}
	},

	/*************************************************************************************************************************
	 * RestartBalloonHelpShow(event) - Restarts the timer to show the balloon help, see ShowBalloonHelpDelayed
	 * event is not used.
	 ************************************************************************************************************************/
	RestartBalloonHelpShow : function(event)
	{
		balloonhelp.debug('Restarting the delay for showing balloon help.');
		balloonhelp.CancelBalloonHelpShow(event);
		balloonhelp.mBalloonHelpShowTimer = setTimeout("balloonhelp.FinalShowBalloonHelp()",balloonhelp.mBalloonHelpDelayTimeout);
		balloonhelp.xPos = event.screenX;
		balloonhelp.yPos = event.screenY;
	},

	/*************************************************************************************************************************
	 * ShowBalloonHelpDelayed(xElementID, inTitle, inDescription, isFeature) - This will save the information needed then
	 * setup a timer to show the balloon help, this can be disabled by CancelBallonHelpShow. This function will attach to
	 * the mouseout event of the incoming element and will cancel if the mouse moves out of the box, it will also attach to
	 * the mousemove event that will restart the timer each time.
	 ************************************************************************************************************************/
	ShowBalloonHelpDelayed : function(inUri, xElementID, inTitle, inDescription, inToolTip, event, inAnchorNode, inXOffset, inYOffset)
	{
		//MERC BH: don't show bubble help if there is already a popup showing
		//fix for BLT#198721
		if(document.popupCurrentlyShowing)
			return;
			
		if (!xElementID)
			return;
			
		if (balloonhelp.mElementID == xElementID)
			return;

		balloonhelp.mElementID = xElementID;
		balloonhelp.mInTitle = inTitle;
		balloonhelp.mInToolTip = inToolTip;
		balloonhelp.mInDescription = inDescription;
		balloonhelp.mIsFeature = false;
		balloonhelp.mIsNested = true;
		balloonhelp.mWidgetPref = "balloon";

		balloonhelp.SetAnchor(xElementID, inAnchorNode);
		balloonhelp.SetOffsets(inXOffset, inYOffset);

		if (event)
		{
			balloonhelp.xPos = event.screenX;
			balloonhelp.yPos = event.screenY;
		}

		if (!balloonhelp.CheckBalloonHelpPrefs()) {
			balloonhelp.mBalloonHelpDelayTimeout = balloonhelp.BALLOON_DELAY_TOOLTIP_TIMEOUT
		} else {
			balloonhelp.mBalloonHelpDelayTimeout = balloonhelp.BALLOON_DELAY_NORMAL_TIMEOUT;
		}

		if (balloonhelp.mBalloonHelpDelayTimeout)
			balloonhelp.mBalloonHelpShowTimer = setTimeout("balloonhelp.FinalShowBalloonHelp()",balloonhelp.mBalloonHelpDelayTimeout);
		else
			balloonhelp.FinalShowBalloonHelp();

		xElementID.addEventListener('mouseout', balloonhelp.CancelBalloonHelpShow, true);
		xElementID.addEventListener('click', balloonhelp.CancelBalloonHelpShow, true);
		xElementID.addEventListener('mousemove', balloonhelp.RestartBalloonHelpShow, true);
	},

	/*************************************************************************************************************************
	 * ShowBalloonHelp(xElementID, inTitle, inDescription, isFeature) - This will save the information needed then
	 * automatically show the balloon using FinalShowBalloonHelp function.
	 ************************************************************************************************************************/
	ShowBalloonHelp : function(inUri, xElementID, inTitle, inDescription, inWidgetPref, inAnchorNode, inXOffset, inYOffset)
	{
		//MERC BH: don't show bubble help if there is already a popup showing
		//fix for BLT#198721
		if(document.popupCurrentlyShowing)
			return;
			
		balloonhelp.mUrlID = inUri;
		balloonhelp.mElementID = xElementID;
		balloonhelp.mInTitle = inTitle;
		balloonhelp.mInDescription = inDescription;
		balloonhelp.mIsFeature = true;
		balloonhelp.mIsNested = false;
		balloonhelp.mWidgetPref = inWidgetPref;
		balloonhelp.SetAnchor(xElementID, inAnchorNode);
		balloonhelp.SetOffsets(inXOffset, inYOffset);

		balloonhelp.mBalloonHelpDelayTimeout = balloonhelp.BALLOON_DELAY_FEATURE_TIMEOUT;

		if (balloonhelp.mBalloonHelpDelayTimeout > 0)
			balloonhelp.mBalloonHelpShowTimer = setTimeout("balloonhelp.FinalShowBalloonHelp()",balloonhelp.mBalloonHelpDelayTimeout);
		else
			balloonhelp.FinalShowBalloonHelp();

		xElementID.addEventListener('mouseout', balloonhelp.CancelBalloonHelpShow, true);
		xElementID.addEventListener('click', balloonhelp.CancelBalloonHelpShow, true);
		xElementID.addEventListener('mousemove', balloonhelp.RestartBalloonHelpShow, true);
	},

	SetAnchor : function (xElementID, inAnchorNode)
	{
		if(inAnchorNode) {
		  this.mAnchorNode = inAnchorNode;
		}
  	else {
  	  this.mAnchorNode = xElementID;
  	}
	},

	SetOffsets : function (inXOffset, inYOffset)
	{
  	if(inXOffset) {
  	  this.mMouseXOffset = inXOffset;
  	}
    else {
      this.mMouseXOffset = 0;
    }

  	if(inYOffset) {
  	  this.mMouseYOffset = inYOffset;
  	}
    else {
      this.mMouseYOffset = 0;
    }
	},

	/*************************************************************************************************************************
	 * isHelpShown() - This will check to see if the balloon is already show
	 * it will allow us to advoid opening multiple popup windows and interupting the user
	 ************************************************************************************************************************/
	isHelpShown : function()
	{
		var myBalloonObj = document.getElementById(this.mBalloonHelpID);
		if (myBalloonObj && myBalloonObj.boxObject.parentBox) return true;

		return false;
	},

	/*************************************************************************************************************************
	 * CreateToolTip() - This will create a normal tool tip.
	 ************************************************************************************************************************/
	CreateToolTip : function(toolTipText, myBalloonID)
	{
		// ToDo: We need to position this like the tooltips (mouseX and mouseY + 16 or 32?)

		var balloonToolTipLabel = document.createElement('label');
		balloonToolTipLabel.setAttribute("class","balloon-tooltip-label");
		balloonToolTipLabel.setAttribute("crop","right");
		balloonToolTipLabel.setAttribute("flex","1");
		balloonToolTipLabel.setAttribute("value",toolTipText);

		myBalloonID.appendChild(balloonToolTipLabel);
	},

	/*************************************************************************************************************************
	 * CreateBalloon() - This will create either a feature or normal balloon bubble.
	 ************************************************************************************************************************/
	CreateBalloon : function(myBalloonID)
	{
			// Here we need to determine where the bubble should be placed
		  this.xPos = this.mElementID.boxObject.screenX + (this.mElementID.boxObject.width / 2) - 5;
		  this.yPos = this.mElementID.boxObject.screenY + (this.mElementID.boxObject.height / 2);

			myBalloonID.setAttribute("isFeature",this.mIsFeature);

			// Set up the new elements

			// This is the top tail (basicaly just a space for the window shaping)
			var balloonTailTop = document.createElement('vbox');
			balloonTailTop.setAttribute("id","balloonTailTop");
			balloonTailTop.setAttribute("class", "balloon-tail");
			balloonTailTop.setAttribute("isFeature",this.mIsFeature);
			balloonTailTop.setAttribute("mousethrough", "always");

			// This is the content (it will be the round rectangle section)
			var balloonContent = document.createElement('vbox');
			balloonContent.setAttribute("id","balloonContent");
			balloonContent.setAttribute("class", "balloon-content");
			balloonContent.setAttribute("isFeature",this.mIsFeature);

				// This is the section between the top tail and Title
				var balloonOptTop = document.createElement('hbox');
				balloonOptTop.setAttribute("id","balloonOptTop");
				balloonOptTop.setAttribute("class", "balloon-extended");
				balloonOptTop.setAttribute("isFeature",this.mIsFeature);

				// Create the Title
				var balloonTitleLabel = document.createElement('label');
				balloonTitleLabel.setAttribute("id","balloonTitle");
				balloonTitleLabel.setAttribute("class","balloon-title");
				balloonTitleLabel.setAttribute("value",this.mInTitle);
				balloonTitleLabel.setAttribute("isFeature",this.mIsFeature);

				// Create the Description
				var balloonDescriptionLabel = document.createElement('description');
				balloonDescriptionLabel.setAttribute("id","balloonDescription");
				balloonDescriptionLabel.setAttribute("class","balloon-description");
				balloonDescriptionLabel.setAttribute("isFeature",this.mIsFeature);
				balloonDescriptionLabel.setAttribute("flex","1");
					// create text node, append to description
					var balloonDescriptionLabelText = document.createTextNode(this.mInDescription);
					balloonDescriptionLabel.appendChild(balloonDescriptionLabelText);

				// This is the section between the tail and Description
				var balloonOptBottom = document.createElement('hbox');
				balloonOptBottom.setAttribute("id","balloonOptBottom");
				balloonOptBottom.setAttribute("class", "balloon-extended");
				balloonOptBottom.setAttribute("isFeature",this.mIsFeature);

			// Bottom Tail
			var balloonTailBottom = document.createElement('vbox');
			balloonTailBottom.setAttribute("id","balloonTailBottom");
			balloonTailBottom.setAttribute("class", "balloon-tail");
			balloonTailBottom.setAttribute("isFeature",this.mIsFeature);

			// Attatch the elements in proper order
			balloonContent.appendChild(balloonOptTop);
			balloonContent.appendChild(balloonTitleLabel);
			balloonContent.appendChild(balloonDescriptionLabel);
			balloonContent.appendChild(balloonOptBottom);

			myBalloonID.appendChild(balloonTailTop);
			myBalloonID.appendChild(balloonContent);
			myBalloonID.appendChild(balloonTailBottom);

			myBalloonID.setAttribute("class","balloon");
	},

	/*************************************************************************************************************************
	 * ReOrganize() - Now that we have created our balloon we can reorganize the contents.
	 * The Help Icon needs to be close to the tail, we also remove any extra spacing created before when
	 * we didn't know the orientation of the balloon.
	 ************************************************************************************************************************/
	ReOrganize : function()
	{
		// Determine how this balloon is to be positioned
		var myBalloonID = document.getElementById(balloonhelp.mBalloonHelpID);
		if (!myBalloonID)
			return;

		var cXPos = balloonhelp.mElementID.boxObject.screenX;
		var cYPos = balloonhelp.mElementID.boxObject.screenY;
		var bXPos = myBalloonID.boxObject.screenX;
		var bYPos = myBalloonID.boxObject.screenY;
		var tailPos = "";

		if (cXPos <= bXPos)
			tailPos += "left";
		else
			tailPos += "right";

		if (cYPos <= bYPos)
			tailPos += "top";
		else
			tailPos += "bottom";

		// Remove unneeded tails
		var balloonTailBottom = document.getElementById("balloonTailBottom");
		var balloonTailTop = document.getElementById("balloonTailTop");

		if (balloonTailBottom && (tailPos.indexOf("top") != -1)) {
				balloonTailBottom.parentNode.removeChild(balloonTailBottom);
		}

		if (balloonTailTop && (tailPos.indexOf("bottom") != -1)) {
				balloonTailTop.parentNode.removeChild(balloonTailTop);
		}

		// This is the image for the help (this may be changed to an achor)
		var balloonHelpImage = document.createElement('image');
		balloonHelpImage.setAttribute("id","balloonHelpImage");
		balloonHelpImage.setAttribute("class", "balloon-help-image");
		balloonHelpImage.setAttribute("src", "chrome://browser/skin/icons/balloonhelp.png");
		balloonHelpImage.setAttribute("onclick", "balloonhelp.ShowHelp();");

		// Just a spacer to make sure the help Icon is on the far left or right.
		var balloonHelpSpace = document.createElement('label');
		balloonHelpSpace.setAttribute("id","balloonHelpSpacer");
		balloonHelpSpace.setAttribute("class", "balloon-help-space");
		balloonHelpSpace.setAttribute("flex", "1");

		// Close button for feature discovery (not on normal balloons)
		var balloonExtendedClose = document.createElement('image');
		balloonExtendedClose.setAttribute("id","balloonExtendedClose");
		balloonExtendedClose.setAttribute("class","balloon-extended-close");
		balloonExtendedClose.setAttribute("src", "chrome://browser/skin/icons/balloonclose.png");
		balloonExtendedClose.setAttribute("onclick", "balloonhelp.HideBalloonHelp();");

		var balloonOptTop = document.getElementById("balloonOptTop");
		var balloonOptBottom = document.getElementById("balloonOptBottom");

		if (balloonOptTop && balloonOptBottom)
		{
			// First do the help and close buttons
			var balloonOpt = null;
			if (tailPos.indexOf("top") != -1) {
				balloonOpt = balloonOptTop;
				if (!this.mIsFeature)
					balloonOptBottom.parentNode.removeChild(balloonOptBottom);
			} else {
				balloonOpt = balloonOptBottom;
				if (!this.mIsFeature)
					balloonOptTop.parentNode.removeChild(balloonOptTop);
			}

			if (tailPos.indexOf("left") != -1)
			{
				// Tail on left side so image first
				balloonOpt.appendChild(balloonHelpImage);
				balloonOpt.appendChild(balloonHelpSpace);
				if (this.mIsFeature)
					balloonOpt.appendChild(balloonExtendedClose);
			} else if(tailPos.indexOf("right") != -1) {
				// Tail on right side so image last
				if (this.mIsFeature)
					balloonOpt.appendChild(balloonExtendedClose);
				balloonOpt.appendChild(balloonHelpSpace);
				balloonOpt.appendChild(balloonHelpImage);
			}

			// Now do the extended buttons (if necessary)
			if (this.mIsFeature)
			{
				if (tailPos.indexOf("top") != -1)
					balloonOpt = balloonOptBottom;
				else
					balloonOpt = balloonOptTop;

				// Check box for feature discovery that disables the balloon popup
				var balloonExtendedCheck = document.createElement('checkbox');
				balloonExtendedCheck.setAttribute("id","balloonExtendedCheck");
				balloonExtendedCheck.setAttribute("class","balloon-extended-check");
				balloonExtendedCheck.setAttribute("checked","true");
				balloonExtendedCheck.setAttribute("label", "Show Tip Next Time");
				balloonExtendedCheck.setAttribute("onclick","balloonhelp.SetFeaturePref('" + this.mUrlID + "',true);");
				balloonExtendedCheck.setAttribute("flex","1");
				balloonExtendedSpace = document.createElement('spacer');

				var balloonExtendedOptions = document.createElement('image');
				balloonExtendedOptions.setAttribute("id","balloonExtendedOptions");
				balloonExtendedOptions.setAttribute("class","balloon-extended-option");
				balloonExtendedOptions.setAttribute("src", "chrome://browser/skin/icons/goto_button.png");
				balloonExtendedOptions.setAttribute("onclick", "balloonhelp.OpenAdvancedOptions();");

				var balloonExtendedOptionsLabel = document.createElement('label');
				balloonExtendedOptionsLabel.setAttribute("id","balloonHelpOptionsLabel");
				balloonExtendedOptionsLabel.setAttribute("class", "balloon-help-space");
				balloonExtendedOptionsLabel.setAttribute("value", this.GetBalloonString("optionsLabel","Options") + " ");

				balloonOpt.appendChild(balloonExtendedCheck);
				balloonOpt.appendChild(balloonExtendedSpace);
				balloonOpt.appendChild(balloonExtendedOptionsLabel);
				balloonOpt.appendChild(balloonExtendedOptions);
			}
		}


		// Fade in after starting the make bubble
		setTimeout("document.getElementById(balloonhelp.mBalloonHelpID).boxObject.MakeBubble('" + tailPos + "')",0);
		this.mBalloonFadeTimer = setTimeout("balloonhelp.BalloonFadeIn(0);",this.TRANSPARENCY_DELAY);
		setTimeout('balloonhelp.SetFullyOpaque();', this.BALLOON_FULLY_VISIBLE_DEADLINE);
		//myBalloonID.boxObject.MakeBubble(tailPos);
	},


	/************************************************************************************************************************
	 * getParentPopupWindow(elementObj) - Gets the parent of the object passed in elementObj
	 ***********************************************************************************************************************/
	getParentPopupWindow : function (elementObj)
	{
		var parentItem = elementObj.parentNode;
		while (parentItem)
		{
			if (parentItem.localName == 'popup')
			{
				this.debug("Found a parentItem called " + parentItem.id + "\n\n");
				return parentItem;
			}
			parentItem = parentItem.parentNode;
		}
		return;
	},

	ClearPopupContents : function(inElementID)
	{
		if (inElementID) {
			// Cycle through all the elements and delete them if they exist
			// This way we don't continuously add new content.
			var item = inElementID.lastChild;
			while (item)
			{
				var next = item.previousSibling;
				item.parentNode.removeChild(item);
				item = next;
			}
		}
	},

	BalloonFadeIn : function(inLevel)
	{
		var inBalloonID = document.getElementById(balloonhelp.mBalloonHelpID);

		if (balloonhelp.mBalloonFadeTimer)
			clearTimeout(balloonhelp.mBalloonFadeTimer);

		if (inBalloonID) {
			if (inLevel < balloonhelp.TRANSPARENCY_MAX && inLevel >= 0) {
				var newLevel = inLevel + balloonhelp.TRANSPARENCY_INC;
				if (newLevel > balloonhelp.TRANSPARENCY_MAX) newLevel = balloonhelp.TRANSPARENCY_MAX;
				inBalloonID.boxObject.setTransparency(newLevel);
				if (newLevel < balloonhelp.TRANSPARENCY_MAX)
					balloonhelp.mBalloonFadeTimer = setTimeout("balloonhelp.BalloonFadeIn(" + newLevel + ");", balloonhelp.TRANSPARENCY_TIMEOUT);
			}
		}
	},

	SetFullyOpaque : function()
	{
		var inBalloonID = document.getElementById(balloonhelp.mBalloonHelpID);

		if (balloonhelp.mBalloonFadeTimer)
			clearTimeout(balloonhelp.mBalloonFadeTimer);
		
		if (inBalloonID) {
				inBalloonID.boxObject.setTransparency(balloonhelp.TRANSPARENCY_MAX);
		}
	},

	/*************************************************************************************************************************
	 * FinalShowBalloonHelp() - Shows the balloon help, either after delay or right away. There has to be a <popup> in a
	 * <popupset> with the ID indicated above this.mBalloonHelpID. This will create new content each time it loads as it will
	 * Change depending on tail position and type.
	 ************************************************************************************************************************/
	FinalShowBalloonHelp : function()
	{
		if(document.popupCurrentlyShowing)
			return;
		
		var myBalloonID = null;
		var toolTipText = "";

		if (this.mElementID)
		{
			this.mElementID.removeEventListener('mouseout', balloonhelp.CancelBalloonHelpShow, true);
			this.mElementID.removeEventListener('click', balloonhelp.CancelBalloonHelpShow, true);
			this.mElementID.removeEventListener('mousemove', balloonhelp.RestartBalloonHelpShow, true);

			var doBalloonShow = true;
			var doToolTipShow = false;

			if (!this.CheckBalloonHelpPrefs()) {
				doBalloonShow = false;
			}

			if (!doBalloonShow && (this.mWidgetPref == 'balloon'))
			{
				// Check for tooltip info
				toolTipText = this.mInToolTip;
				if (toolTipText) {
					doToolTipShow = true;
				}
			}

			// Anything to show?
			if (!doBalloonShow && !doToolTipShow)
				return;

			var normalBalloonObj = document.getElementById(this.mBalloonHelpID);
			var tooltipBalloonObj = document.getElementById(this.mBalloonHelpID + "ToolTip");

			// Stop any old Balloon help from showing and hide them if visible.
			this.PauseTimer();

			if (normalBalloonObj) {
				if(document.popupCurrentlyShowing)
					return;
				normalBalloonObj.hidePopup();
				this.ClearPopupContents(normalBalloonObj);
			}

			if (tooltipBalloonObj) {
				if(document.popupCurrentlyShowing)
					return;
				tooltipBalloonObj.hidePopup();
				this.ClearPopupContents(tooltipBalloonObj);
			}
			
			if (doToolTipShow)
				this.mBalloonHelpTimeout = this.BALLOON_TOOLTIP_TIMEOUT;
				

			if (!doToolTipShow)
				myBalloonID = normalBalloonObj;
			else
				myBalloonID = tooltipBalloonObj;

			if (doBalloonShow) {
				this.CreateBalloon(myBalloonID);
			} else {
				this.CreateToolTip(toolTipText,myBalloonID);
			}

			if (myBalloonID)
			{
				// Show the balloonhelp
				if (!doToolTipShow) {
					var parentPopup = this.getParentPopupWindow(this.mElementID);
					if (parentPopup)
					{
						// Normal balloon help
						this.xPos = ((this.mElementID.boxObject.width / 2) - 5) + this.mElementID.boxObject.screenX;
						this.yPos = (this.mElementID.boxObject.height / 2) + this.mElementID.boxObject.screenY;
						if(document.popupCurrentlyShowing)
							return;
						document.balloonCurrentlyShowing = true;
						myBalloonID.showPopup(this.mAnchorNode, this.xPos - parentPopup.boxObject.screenX, this.yPos - parentPopup.boxObject.screenY, 'tooltip');
						if (this.mIsFeature) { myBalloonID.securePopup(true); } // Prevent the balloon from dissapearing when the user clicks else where
						myBalloonID.boxObject.setTransparency(0);
						myBalloonID.moveTo(this.xPos, this.yPos);
						setTimeout("balloonhelp.ReOrganize()", 0);
					} else {
						// Tooltip balloon help
						if ( (this.xPos <= 0) && (this.yPos <= 0))
							return;
						if(document.popupCurrentlyShowing)
							return;
						document.balloonCurrentlyShowing = true;
						myBalloonID.showPopup(this.mAnchorNode, this.xPos + this.mMouseXOffset, this.yPos + this.mMouseYOffset, 'popup');
						if (this.mIsFeature) { myBalloonID.securePopup(true); } // Prevent the balloon from dissapearing when the user clicks else where
						myBalloonID.boxObject.setTransparency(0);
						// We need to reorganize the balloon after it pops up.
						setTimeout("balloonhelp.ReOrganize()", 0);
					}
				} else {
					var parentPopup = this.getParentPopupWindow(this.mElementID);
					if (parentPopup)
					{
						if(document.popupCurrentlyShowing)
							return;
						document.balloonCurrentlyShowing = true;
						myBalloonID.showPopup(this.mAnchorNode, this.xPos + balloonhelp.mMouseXOffset, this.yPos + balloonhelp.mMouseYOffset, 'tooltip');
						this.mElementID.addEventListener('mousemove', balloonhelp.HideBalloonHelp, true);
					} else {
						// This shows up in the wrong spot? Why? We want it below the mouse pointer
						if(document.popupCurrentlyShowing)
							return;
						document.balloonCurrentlyShowing = true;
						myBalloonID.showPopup(this.mAnchorNode, this.xPos + balloonhelp.mMouseXOffset, this.yPos + balloonhelp.mMouseYOffset, 'popup');
						this.mElementID.addEventListener('mousemove', balloonhelp.HideBalloonHelp, true);
					}
				}
				
				// Attach listeners for mouseover/mouseout events to pause/start the timer for closing the balloonhelp
				if (doBalloonShow)
				{
					if (this.mIsFeature)
					{
						this.mBalloonHelpTimeout = this.BALLOON_FEATURE_TIMEOUT;
					} else {
						this.mBalloonHelpTimeout = this.BALLOON_NORMAL_TIMEOUT;
						myBalloonID.addEventListener('click', this.HideBalloonHelp, true);	// This will hide the balloon if the user clicks in the balloon
					}
				}

				myBalloonID.addEventListener('mouseover', this.PauseTimer, true);	// This will stop the timer when the user hovers over the balloon
				myBalloonID.addEventListener('mouseout', this.StartTimer, true);		// This will start the timer when the user leave the balloon
				this.mElementID.addEventListener('click', balloonhelp.HideBalloonHelp, true);

				// Start the timer by default to close the balloon after an apropriate delay
				this.StartTimer();

			}
		}
	},

	/*************************************************************************************************************************
	 * These are the Timer or Event functions, they are special in that instead of using "this" you have to use balloonhelp as
	 * the object, everything else above will generally still use "this". (Scope issues)
	 ************************************************************************************************************************/

	/*************************************************************************************************************************
	 * openAdvancedOptions(aEvent) - This will open the preferences dialog and navigate to the advanced page
	 ************************************************************************************************************************/
	OpenAdvancedOptions : function(aEvent)
	{
		var PREF_ADVANCED=3;
		balloonhelp.HideBalloonHelp(aEvent);
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

		prefs.setIntPref('browser.preferences.lastpanel',PREF_ADVANCED);
		openDialog("chrome://browser/content/pref/pref.xul","PrefWindow",
		   "chrome,titlebar,resizable=no,modal", false, 'catAdvancedButton');
	},

	/*************************************************************************************************************************
	 * CleanUpBalloonHelp(balloonHelpID) - Cleans up the balloon help after closing/hiding
	 ************************************************************************************************************************/
	CleanUpBalloonHelp : function(balloonHelpID)
	{
		
		if (balloonHelpID) {
			balloonHelpID.removeEventListener('mouseout', balloonhelp.CancelBalloonHelpShow, true);
			balloonHelpID.removeEventListener('mouseover', balloonhelp.CancelBalloonHelpShow, true);
			balloonHelpID.removeEventListener('click', balloonhelp.CancelBalloonHelpShow, true);
			balloonHelpID.removeEventListener('click', this.HideBalloonHelp, true);
			balloonHelpID.removeEventListener('mouseover', this.PauseTimer, true);
			balloonHelpID.removeEventListener('mouseout', this.StartTimer, true);
		}
		
		if (balloonhelp.mElementID) {
			// Remove any listeners we have created (this will not do anything if we didn't create any)
			balloonhelp.mElementID.removeEventListener('mouseout', balloonhelp.HideBalloonHelp, true);
			balloonhelp.mElementID.removeEventListener('mouseout', balloonhelp.CancelBalloonHelpShow, true);
			balloonhelp.mElementID.removeEventListener('click', balloonhelp.CancelBalloonHelpShow, true);
			balloonhelp.mElementID.removeEventListener('mousemove', balloonhelp.RestartBalloonHelpShow, true);
			balloonhelp.mElementID.removeEventListener('mousemove', balloonhelp.HideBalloonHelp, true);

			if (balloonhelp.mBalloonFadeTimer)
				clearTimeout(balloonhelp.mBalloonFadeTimer);
			//balloonhelp.mElementID = null;
			
		}
	},

	/*************************************************************************************************************************
	 * hideBalloonHelp(aEvent) - hides the balloon if it is shown. aEvent is the event object from events.
	 ************************************************************************************************************************/
	HideBalloonHelp : function(aEvent)
	{
		balloonhelp.debug('Hiding balloon help.');

		// First check the normal balloon and destory
		var balloonHelpID = document.getElementById(balloonhelp.mBalloonHelpID);
		if (balloonHelpID)
		{
			balloonHelpID.hidePopup();
			if (balloonHelpID.hasAttribute('left')){
				balloonhelp.debug("removing left attribute");
				balloonHelpID.removeAttribute('left');
			}
			if (balloonHelpID.hasAttribute('top')){
				balloonhelp.debug("removing top attribute");
				balloonHelpID.removeAttribute('top');
			}
		}

		// Now check the tooltip one and destroy
		balloonHelpID = document.getElementById(balloonhelp.mBalloonHelpID + 'ToolTip');
		if (balloonHelpID)
		{
			balloonHelpID.hidePopup();
		}
	},

	/*************************************************************************************************************************
	 * pauseTimer(aEvent) - This will pause the timer that hides the balloon, usually if the user moves the mouse over it.
	 ************************************************************************************************************************/
	PauseTimer : function(aEvent)
	{
		balloonhelp.debug('Pausing Hide balloon help timer.');
		clearTimeout(balloonhelp.mBaloonHelpTimer);
	},

	/*************************************************************************************************************************
	 * startTimer(aEvent) - This will start the timer that hides the balloon, happens on start and on mouse out.
	 ************************************************************************************************************************/
	StartTimer : function(aEvent)
	{
		balloonhelp.PauseTimer(aEvent);
		balloonhelp.debug('Starting Hide balloon help timer.');
		balloonhelp.mBaloonHelpTimer = setTimeout("balloonhelp.HideBalloonHelp()", balloonhelp.mBalloonHelpTimeout);
	},

	/*************************************************************************************************************************
	 * getBrowser() - this will locate the navigator window and then find the browser content element
	 ************************************************************************************************************************/
	GetBrowser : function()
	{
		var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
		var windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
		var browserWindow = windowManagerInterface.getMostRecentWindow( "navigator:browser" );

		if (browserWindow) {
			try {
				var browser = browserWindow.document.getElementById("content");
				if (browser)
					return browser;
			} catch (err) {
				dump("Unable to load browser content: " + err + "\n");
			}
		}
		return null;
	},

	/*************************************************************************************************************************
	 * showHelp() - activates the help for balloon tips if the user clicks the help icon.
	 ************************************************************************************************************************/
	ShowHelp : function()
	{
		balloonhelp.debug('Showing help for balloonhelp.');
		balloonhelp.PauseTimer();
		balloonhelp.HideBalloonHelp();
		// Here we have to load a url for help (but where do we go?)
		try {
			loadURI("http://browser.netscape.com/ns8/help/default.jsp", null, null);
		} catch (err) {
			// Try to get the browser window and load the url
			var browser = balloonhelp.GetBrowser();
			if (browser && browser.loadURI)
				browser.loadURI("http://browser.netscape.com/ns8/help/default.jsp", null, null);
		}
	},

}
