// pageBlocking.js

// CONSTANTS
const DEBUG = true;

// GLOBALS
var gProfileName;
var gBlockedSite;
var gBundle;
var gProfileMgr;
var gParentalCtrlSvc;
var debug;
var promptService;

if (!DEBUG)
{
  debug = function(msg) {};
}
else
{
  debug = function(msg)
  {
    dump("~~~ pageBlocking.js: " + msg + "\n");
  };
}

/*
*****************************************************************
* Function:  Init
* Arguments: None
* Return: None
* Description: Performs initialization code for startup routine
* Author: Ryan Fransen
*****************************************************************
*/
function Init()
{
	gBundle = document.getElementById("pageBlockingBundle");
	var argsArray = getArgsArray();
	gBlockedSite = unescape(argsArray['hostname']);
	
	try
	{
		gProfileMgr = Components.classes["@mozilla.org/toolkit/profile-service;1"]
										.getService(Components.interfaces.nsIToolkitProfileService);
										
	  gParentalCtrlSvc = Components.classes["@mozilla.org/browser/parentalcontrols-service;1"]
	                        .getService(Components.interfaces.nsIParentalControlsService);
	}
	catch(e)
	{
		debug("Failed creating Provile Manager service. " + e.message);
		try
		{
			PopulateTextFields();
			document.getElementById("adminOptionsLink").hidden = true;
			document.getElementById("blockedsite.buttons.continue").hidden = true;
			
		}
		catch(e) {debug(e.message);}
		return;
	}
	
	try
	{
		promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
	                    .getService(Components.interfaces.nsIPromptService);
	}
	catch(e)
	{
		debug("Failed creating Prompt Service. " + e.message);
		try
		{
			PopulateTextFields();
		}
		catch(e) {debug(e.message);}
		return;
	}
	
	gProfileName = getcurrentProfileName()
	try
	{
		PopulateTextFields();
	}
	catch(e) {debug(e.message);}
	PopulateAdminNamesPopup();
}

/*
*****************************************************************
* Function:  getcurrentProfileName
* Arguments: None
* Return: string
* Description: Returns the name (displayName) of the currently
* 	logged-in profile.
* Author: Ryan Fransen
*****************************************************************
*/
function getcurrentProfileName()
{ 
	try
	{
		var currentProfile = gProfileMgr.getProfileByName(gProfileMgr.selectedProfileName);
		if (currentProfile)
		{
			currentProfile.QueryInterface(Components.interfaces.nsIToolkitProfile);
			return currentProfile.displayName;
		}
		else
		{
			return "";
		}
	}
	catch(e)
	{
		this.debug("getcurrentProfileName failed: " + e.message );
		return "";
	}					
}

/*
*****************************************************************
* Function:  getArgsArray
* Arguments: None
* Return: Array
* Description: Returns an array of the querystring paramaters
* 	and the associated values
* Author: Ryan Fransen
*****************************************************************
*/
function getArgsArray() {
	var qInd = document.location.href.indexOf('?');
	var args = document.location.href.substring(qInd+1);
	var argsList = args.split("&");
	var argsArray = new Array();
	for (var i = 0; i < argsList.length; i++) {
		var argPair = argsList[i].split('=');
		argsArray[argPair[0]] = argPair[1];
	}
	return argsArray;
}

/*
*****************************************************************
* Function:  PopulateTextFields
* Arguments: None
* Return: None
* Description: Populates the text in the page with fields from
* 	the .properties file via the xul string bundle.
* Author: Ryan Fransen
*****************************************************************
*/
function PopulateTextFields()
{
  var blockedSite = gBlockedSite;
  if (blockedSite.length > 60)
  {
    blockedSite = blockedSite.substring(0,59)+"..";
  }
	var btnContinueText = gBundle.getString("blockedsite.buttons.continue");
	var btnBackText = gBundle.getString("blockedsite.buttons.cancel");
	document.getElementById("blockedsite.title").value = unescape(gBundle.getString("blockedsite.title"));
	document.getElementById("blockedsite.description").appendChild(document.createTextNode(unescape(gBundle.getFormattedString("blockedsite.description", [blockedSite, btnBackText]))));
	document.getElementById("blockedsite.adminoptions.label").value = unescape(gBundle.getString("blockedsite.adminoptions.label"));
	document.getElementById("blockedsite.adminoptions.description").appendChild(document.createTextNode(unescape(gBundle.getFormattedString("blockedsite.adminoptions.description", [gProfileName, gBlockedSite, btnContinueText]))));
	document.getElementById("blockedsite.adminoptions.name.label").value = unescape(gBundle.getString("blockedsite.adminoptions.name.label"));
	document.getElementById("blockedsite.adminoptions.password.label").value = unescape(gBundle.getString("blockedsite.adminoptions.password.label"));
	document.getElementById("blockedsite.adminoptions.persistbox.label").label = unescape(gBundle.getFormattedString("blockedsite.adminoptions.persistbox.label", [gProfileName]));
}

/*
*****************************************************************
* Function: PopulateAdminNamesPopup
* Arguments: None
* Return: None
* Description: Populates the admin names popup with all of the
* 	registered browser admin profile names.
* Author: Ryan Fransen
*****************************************************************
*/
function PopulateAdminNamesPopup()
{
	debug('PopulateAdminNamesPopup() start');
	var popup = document.getElementById("adminNamesPopup"); 
	try
	{
		var profiles = gProfileMgr.profiles;
	}
	catch(e)
	{
		debug("getProfiles failed" + e.message );
	}			
	
	while (profiles.hasMoreElements())
 	{
 		var profile = profiles.getNext().QueryInterface(Components.interfaces.nsIToolkitProfile);
		if (profile.isAdmin)
		{
			var item = document.createElement('menuitem');
			item.setAttribute("class", "");			    	
			item.setAttribute('label', profile.displayName);
			item.setAttribute('id', profile.name);
			item.setAttribute('value', profile.name);
			popup.appendChild(item);
		}
	}
	
	// Select the first entry in the list
	var list = document.getElementById("admin-names-text");
	list.selectedItem = list.firstChild.childNodes[0];
	
	debug('PopulateAdminNamesPopup() complete');
}

/*
*****************************************************************
* Function:  Continue
* Arguments: None
* Return: None
* Description: Event Handler for the OnClick event of the 'continue'
* 	button.  Attempts to authorize the aupplied credentials and 
* 	if successful will go ahead and load the blocked page.
* Author: Ryan Fransen
*****************************************************************
*/
function Continue()
{
	var adminctl = document.getElementById("admin-names-text").selectedItem;
	var admin;
	var adminfriendly;
	
	if (adminctl)
	{
		admin = adminctl.getAttribute('value');
		adminfriendly = adminctl.getAttribute('label');
	}
	
	var password = document.getElementById("adminPassword").value;
	if (!password)
	{
		password = "";
	}
	
	if (admin)
	{
		// Always allow this site?
		if (document.getElementById("blockedsite.adminoptions.persistbox.label").checked)
		{
			// Always allow this profile access to this site.
			try
			{
			  dump('AAM: allow site permanently\n');
  			gParentalCtrlSvc.adminAddAllowedSite(gBlockedSite, admin, password, false);
			}
			catch(e)
			{
			  dump('AAM: allow site for this session\n');
				// Fail gracefully, for retry
				debug("Exeption during site pref configuration. " + e.message);
				// Inform the user to try again
				promptService.alert(null, gBundle.getString("blockedsite.authentication.failure.title"), gBundle.getString("blockedsite.authentication.failure.message"));
				return;
			}
		}
		else
		{
			// Only allow this change for this session
			try
			{
  			gParentalCtrlSvc.adminAddAllowedSite(gBlockedSite, admin, password, true);
			}
			catch(e)
			{
				// Fail gracefully, for retry
				debug("Exeption during site pref configuration. " + e.message);
				promptService.alert(null, gBundle.getString("blockedsite.authentication.failure.title"), gBundle.getString("blockedsite.authentication.failure.message"));
				return;
			}
		}
		// Reload site
		document.location.href = gBlockedSite;
	}
	else
	{
		// Not sure what to do here.  We should never get here...  To be here means that 
		// there is no item selected in the admins select box.
		debug("Unable to determine admin from the list select box");
	}
}

/*
*****************************************************************
* Function:  Cancel
* Arguments: None
* Return: None
* Description: Event handler for the OnClick event of the 'Cancel'
* 	button.  auses the browser to go back one page in history.
* Author: Ryan Fransen
*****************************************************************
*/
function Cancel()
{
	history.go(-1);
}

/*
*****************************************************************
* Function: ToggleAdminOptionsBox
* Arguments: None
* Return: None
* Description: Toggles the visibility of the admin options
* Author: Ryan Fransen
*****************************************************************
*/
function ToggleAdminOptionsBox()
{
	var adminOptionsBox = document.getElementById("adminOptionsBox");
	var adminOptionsLinkImg = document.getElementById("adminOptionsLinkImg");
	
	if (adminOptionsBox.hidden)
	{
		adminOptionsBox.hidden = false;
		adminOptionsLinkImg.src = "chrome://browser/content/pageBlocking/up.png";
	}
	else
	{
		adminOptionsBox.hidden = true;
		adminOptionsLinkImg.src = "chrome://browser/content/pageBlocking/down.png";
	}
	
	document.getElementById("blockedsite.buttons.continue").disabled = adminOptionsBox.hidden;
}

