const C = Components.classes;
const I = Components.interfaces;

const ToolkitProfileService = "@mozilla.org/toolkit/profile-service;1";
const PromptService = "@mozilla.org/embedcomp/prompt-service;1";

var gPromptService;
var gGlobalSettingsBox;
var gSelectorView;
var gMyProfile; 
var gDeckMain;
var gLstParentalControlMode;
var gBoxOtherProfilesContainer;
var gViewListItemGlobalSettings;
var gLblSelectionIndicatorPart1;
var gLblSelectionIndicatorPart2;
var gTabParentalControls;
var gPnlPrntlCntrls;
var gProfileSettingsView;
var gTabProfileSettings;
var gProfiles;
var gLoggedInProfile;
var DEBUG = true;
var gSelectedProfile;
var gAutoLoginMenulist;
var gUserLoginRadioGroup;
var dDeckProfileOperations;
var gShowProfileCreationButtonCheck;
var gAllowNewProfilesByAnyone;
var gAllProfilesNeedPassword;
var gAllProfilesNeedPasswordToLogin;
var gAddProfileButton;
var gBundleMain;
var gMasterListTreeListener;
var gGlobalBlockListTreeListener;
var gGlobalAllowListTreeListener;
var gTreeProfileSiteList;
var gTreeProfileSiteList;
var gTreeGlobalBlockList;
var gTreeGlobalAllowList;
var profileUtils;

var gPC_TYPE;
var gPC_NAME;
var gPC_LIST_TYPE;
var gPC_PERMIT_TYPE;
var gPC_PROVIDER_ID;

const ACTION_ADD_PROFILE = "ACTION_ADD_PROFILE";
const REMOVE_PROFILE = "REMOVE_PROFILE";

function debug(msg){
	if (msg && DEBUG) {
		dump('pref-profile.js : ' + msg + '\n')
	}
}


function getSelectedProfile() {
	return gSelectorView.selected.profile;
}


function doLayout()
{	
	debug('doLayout()');	
	
	//Profile item selected
	if (gSelectorView.selected.localName == "profile") {  	  	    	
  
  	if (gSelectorView.selected == gMyProfile){
  		// My profile is selected
  		gLblSelectionIndicatorPart1.value = 'My Profile Settings';  		
  		gLblSelectionIndicatorPart2.hidden = true;  		
  	} else {
  		gLblSelectionIndicatorPart2.hidden = false;
  		gLblSelectionIndicatorPart1.value = "Settings for ";
  		gLblSelectionIndicatorPart2.value = gSelectorView.selected.profile.displayName;  		  		
  	}  	
  	 	
   	// Global settings for admin user only  	
  	if (gLoggedInProfile.isAdmin == true) {  		
  		gViewListItemGlobalSettings.hidden = false;
  		document.getElementById('boxEditMstrPrntlCtrls').hidden = false;
  		gDeckMain.selectedIndex = 1;	
  	}else {  		  		
  		gViewListItemGlobalSettings.hidden = true;
  		document.getElementById('boxEditMstrPrntlCtrls').hidden = true;
  		gDeckMain.selectedIndex = 0;	
  	}
  	  	
  	// Selected profile is an admin so hide parental controls
		if (gSelectorView.selected.profile.isAdmin == true) {											
				  gPnlPrntlCntrls.collapsed = true;				
				  gTabParentalControls.collapsed = true;								  
				  gTabProfileSettings.selectedIndex = 0;
		} else {				
				  gPnlPrntlCntrls.collapsed = false;				
				  gTabParentalControls.collapsed = false;								 
  	}
  	
  	// Set control mode dependant labels
  	document.getElementById('lblListFor').value = gSelectedProfile.displayName;						  	
  	debug('current profile blocking mode:' + gSelectedProfile.blockingMode);
		if (gSelectedProfile.blockingMode == Components.interfaces.nsIParentalControlsService.PC_MODE_BLOCK) {			
			debug('showing blocking mode');
			document.getElementById('lblSiteListDescriptorPart1').value = 'Blocked Sites for ';
			document.getElementById('lblSiteListDescriptorPart2').value = gSelectedProfile.displayName
  		gDeckProfileOperations.selectedIndex = 0;
  	} else {
  		debug('showing allowing mode');
  		document.getElementById('lblSiteListDescriptorPart1').value = 'Allowed Sites for '
			document.getElementById('lblSiteListDescriptorPart2').value = gSelectedProfile.displayName
  		gDeckProfileOperations.selectedIndex = 1;
  	}	
  }
  // Global settings selected
  else if (isGlobalSettingsSelected()) {  	   	   	
  	showGlobalSettings();
  }
  
  
  //*
  //* logic for enable/disable remove profile button
  //*
  var btnRemoveProfile = document.getElementById("btnRemoveProfile");
  if(gLoggedInProfile.isAdmin == true) {
  	// show remove profile button
  	btnRemoveProfile.hidden = false;

		// if admin selects a profile which is not their own		
	  if(gSelectorView.selected && (	gSelectorView.selected.localName == "profile") && (gLoggedInProfile !== gSelectorView.selected.profile)){
	  	// enable remove profile button
	  	btnRemoveProfile.disabled = false;	  	
	  } else{
	  	// disable remove profile button
	  	btnRemoveProfile.disabled = true;
		}		
		
	// non admin
	} else {
		
  	// hide remove profile button  	
  	btnRemoveProfile.disabled = true;
  	if(profileUtils.getAllowNewProfileCreation()){
  		gAddProfileButton.disabled = false;
  	}else {
  		gAddProfileButton.disabled = true;
  	}
	}
	
	manageSiteListButtons();
	
	// If the "Require All Profiles to HAve a Password" option is checked then
	// enable the "Require All Users to Use a Password to Log In"
	gAllProfilesNeedPasswordToLogin.disabled = !gAllProfilesNeedPassword.checked;    
	
	// Disable the auto login list until the auto login option has been
	// selected
	gAutoLoginMenulist.disabled = gUserLoginRadioGroup.value != "AutoLogin"	
	
	// save allowNewProfilesByAnyone
	if(profileUtils.getAllowNewProfileCreation()){		
		gShowProfileCreationButtonCheck.disabled = false;
	}else {
		gShowProfileCreationButtonCheck.disabled = true;		
	}
	
	// enable/disable "Require All Users to User a Password to Log In" checkbox
	if(profileUtils.getAllProfilesNeedPassword()){
		gAllProfilesNeedPasswordToLogin.disabled = false;
	} else {
		gAllProfilesNeedPasswordToLogin.disabled = true;	
	}
			
}


function manageSiteListButtons(){
	
	debug('manageSiteListButtons()');	

	// The remove button is only available when there is at least on tree with a selection
	if (document.getElementById('treeGlobalAllowList').currentIndex == -1 && 
			document.getElementById('treeGlobalBlockList').currentIndex == -1) {
			document.getElementById('btnRemoveSiteFromGlobal').disabled = true; 
	} else {
			document.getElementById('btnRemoveSiteFromGlobal').disabled = false; 	
	}
	
	// Manage the state of the remove button for the user list
	var treeProfileSiteList = document.getElementById("treeProfileSiteList");
	document.getElementById('btnRemoveBlockUser').disabled = true; 
	document.getElementById('btnRemoveAllowUser').disabled = true; 

	try {				
		if (treeProfileSiteList.currentIndex > -1) {
								
			var ds = getDatasourceFromTree(treeProfileSiteList); 		
			var start = new Object();
			var end = new Object();
			var numRanges = treeProfileSiteList.view.selection.getRangeCount();
			// section deals with mutiple selection
			var removeSite = true;
			for (var t=0; t<numRanges; t++){	
				treeProfileSiteList.view.selection.getRangeAt(t,start,end);			
				if(removeSite == true)
					{
							for (var v=start.value; v<=end.value; v++)
							{
												
								var res = gTreeProfileSiteList.builder.getResourceAtIndex(v);			
								
								if (ds.GetTarget(res, 
										profileUtils.RDF_SVC.GetResource('http://home.netscape.com/PC-rdf#ListType'),
										true).
										QueryInterface(Components.interfaces.nsIRDFLiteral).Value != 'GlobalList')
										{			
										// web site selected doesn't belong to the Master list		
										document.getElementById('btnRemoveBlockUser').disabled = false; 	
										document.getElementById('btnRemoveAllowUser').disabled = false;
										 	
										}else
										{
												
												// web site selected belongs to the Master list
												document.getElementById('btnRemoveBlockUser').disabled = true; 	
												document.getElementById('btnRemoveAllowUser').disabled = true;
												removeSite = false;
												break;
										}
							}
				}
			}
		} 
	}catch (e) {
		// Swallow exception for the case when rdf resource items are deleted.
	}
		
}

// Programmatically select the global settings option
function showGlobalSettings(tabIndexToSelect){
	
	gLblSelectionIndicatorPart2.hidden = true;
	gLblSelectionIndicatorPart1.value = "Global Settings";
		
	// Only select the global settings item from the view if it has not been
	// set
	if (gSelectorView.selected != document.getElementById('viewListItemGlobalSettings')){
		gSelectorView.selected = document.getElementById('viewListItemGlobalSettings');			
	}	
	gDeckMain.selectedIndex = 2;	
	
	// Allow the caller to explicitly select a tab
	// 0 = Global settings 1 = Master parental controls
	if (tabIndexToSelect){
		document.getElementById('tabGlobalSettings').selectedIndex = tabIndexToSelect;		
	}
}

function viewListItemPreSelected()
{
	debug('viewListItemPreSelected()');	
	if(!gProfileSettingsView.validate())
	{
		debug('stopping selection as validation failed...');	
		gSelectorView.stopped = true;
	}
}

function viewListItemSelected() {
	
	debug('viewListItemSelected()');
	// validate before you allow add profile
	if(!gProfileSettingsView.validate())
	{
			debug('returning as validation failed...');	
			return;	
	}
		
	if (gSelectorView.selected.localName == "profile") 
	{
		saveForm();				
		gSelectedProfile = gSelectorView.selected.profile;
		bindForm();
	}
		doLayout();

}

function getParentalControlMode(){

	if 	(gLstParentalControlMode.selectedItem) return gLstParentalControlMode.selectedItem.value
	return null;
	
}


function bindForm(){
	debug('bindForm()');	
	gProfileSettingsView.profile = gSelectedProfile;		
	gLstParentalControlMode.value = gSelectedProfile.blockingMode;
	bindPrntlCtrlLists();		
	gProfileSettingsView.bind();		
					
	if (profileUtils.getDefaultProfile() == ''){
		gUserLoginRadioGroup.value == "NoAutoLogin";
		gUserLoginRadioGroup.selectedIndex = 0;		
		gAutoLoginMenulist.selectedIndex = 0;
	} else {
		gUserLoginRadioGroup.value == "AutoLogin";
		gUserLoginRadioGroup.selectedIndex = 1;
		gAutoLoginMenulist.value = profileUtils.getDefaultProfile();		
	}	
	
	gAllowNewProfilesByAnyone.checked = profileUtils.getAllowNewProfileCreation();		
	gShowProfileCreationButtonCheck.checked = profileUtils.getShowProfileCreationButton();	
	gAllProfilesNeedPassword.checked = profileUtils.getAllProfilesNeedPassword();
	gAllProfilesNeedPasswordToLogin.checked = profileUtils.getAllProfilesNeedPasswordToLogin();
			
}

function saveForm(){
	debug('saveForm()');	
	gProfileSettingsView.save();		
	gSelectedProfile.blockingMode = getParentalControlMode();
		
	// Save auto login settings
	if(gUserLoginRadioGroup.value == "NoAutoLogin") {
		profileUtils.setDefaultProfile("");
		profileUtils.setStartWithLastProfile(false);		
	} else {		
		profileUtils.setDefaultProfile(gAutoLoginMenulist.value);
		profileUtils.setStartWithLastProfile(true);
	}
	
	profileUtils.setAllowNewProfileCreation(gAllowNewProfilesByAnyone.checked);
	
	if(!profileUtils.getAllowNewProfileCreation()){
		profileUtils.setShowProfileCreationButton(false);
	} else {
		profileUtils.setShowProfileCreationButton(gShowProfileCreationButtonCheck.checked);
	}
	
	profileUtils.setAllProfilesNeedPassword(gAllProfilesNeedPassword.checked);	
	if(!profileUtils.getAllProfilesNeedPassword()){
		profileUtils.setAllProfilesNeedPasswordToLogin(false);
	} else {
		profileUtils.setAllProfilesNeedPasswordToLogin(gAllProfilesNeedPasswordToLogin.checked);
	}									  		
}

function profileSettingsChanged() {
	debug('profileSettingsChanged()');	
	gProfileSettingsView.save();		  
	gProfileSettingsView = getProfileSettingsView();		
	gProfileSettingsView.profile = gSelectorView.selected.profile;	
	gProfileSettingsView.bind();
	refreshProfileSelector();	
	refreshAutoLoginList();	
	doLayout();			
}

function isGlobalSettingsSelected () {

	return gSelectorView.selected.id == "viewListItemGlobalSettings";
}
	
function StartUp()
{
  try {
  	
		if (!parent.profileData) {
			parent.profileData = {
				profileUtils : null,
				isDataValid : false				
			}
		}
							
		// If we have cached data use it otherwise
		// refresh from file
		if (parent.profileData.profileUtils){						
			profileUtils = parent.profileData.profileUtils;				
		} else {
			profileUtils = new objProfileManager();
			profileUtils.initServices();
			profileUtils.refreshProfiles();  		
  	}
  	
  	// BLT 211334: no displayName defined in 8.0.4
  	var profileList = profileUtils.getProfiles();		
	while (profileList.hasMoreElements()) {
		var profile = profileList.getNext().QueryInterface(Components.interfaces.nsIToolkitProfile);
		if(profile.displayName.length == 0) {
			profile.displayName = profile.name;
		}
	}
  	
  	gPC_TYPE = profileUtils.RDF_SVC.GetResource('http://home.netscape.com/PC-rdf#Type'); //URISpec or Provider
		gPC_NAME = profileUtils.RDF_SVC.GetResource('http://home.netscape.com/PC-rdf#Name');
		gPC_LIST_TYPE = profileUtils.RDF_SVC.GetResource('http://home.netscape.com/PC-rdf#ListType'); //GlobalList or UserList
		gPC_PERMIT_TYPE = profileUtils.RDF_SVC.GetResource('http://home.netscape.com/PC-rdf#PermitType'); //Block or Allow		
		gPC_PROVIDER_ID = profileUtils.RDF_SVC.GetResource('http://home.netscape.com/PC-rdf#providerId');
  			
		// Flag parent data as invalid.
		parent.profileData.isDataValid = false;	
		
	  gTreeProfileSiteList = document.getElementById("treeProfileSiteList");
		gTreeGlobalBlockList = document.getElementById("treeGlobalBlockList");
		gTreeGlobalAllowList = document.getElementById("treeGlobalAllowList");  	
  	gAutoLoginMenulist = document.getElementById("AutoLoginMenuList");
  	gBundleMain = document.getElementById("bundle_PrefProfile");  	
  	document.getElementById("profileNameBroadCaster").addEventListener("DOMAttrModified", onProfileNameBroadCasterAttribChanged, false);  
  	// Get references to commonly used controls for convenience
  	gDeckMain = document.getElementById("deckMain");  	
  	gLblSiteListDescriptor = document.getElementById("lblSiteListDescriptor");
  	gSelectorView = document.getElementById("selectorView");
  	gSelectorView.addEventListener("richview-select", viewListItemSelected, false);
  	gSelectorView.addEventListener("richview-pre-select", viewListItemPreSelected, false);
  	gMyProfile = document.getElementById("myProfile");
  	gLstParentalControlMode = document.getElementById("lstParentalControlMode");
  	gBoxOtherProfilesContainer =  document.getElementById("boxOtherProfilesContainer");
  	gDeckProfileOperations = document.getElementById("deckProfileOperations");
  	gViewListItemGlobalSettings = document.getElementById("viewListItemGlobalSettings");
  	gTabParentalControls = document.getElementById("tabParentalControls");
  	gPnlPrntlCntrls = document.getElementById("pnlPrntlCntrls");
  	gTabProfileSettings = document.getElementById("tabProfileSettings");  	
  	gLblSelectionIndicatorPart1 = document.getElementById("lblSelectionIndicatorPart1");
  	gLblSelectionIndicatorPart2 = document.getElementById("lblSelectionIndicatorPart2");
  	gUserLoginRadioGroup = document.getElementById("UserLoginArea");
  	gShowProfileCreationButtonCheck = document.getElementById("ShowProfileCreationButtonCheck");
  	gAllowNewProfilesByAnyone = document.getElementById("allowNewProfilesByAnyone");
  	gAllProfilesNeedPassword = document.getElementById("allProfilesNeedPassword");
  	gAllProfilesNeedPasswordToLogin = document.getElementById("allProfilesNeedPasswordToLogin");
  	gAddProfileButton = document.getElementById("AddProfileButton");
  	gLoggedInProfile = profileUtils.getLoggedInProfile();  	
  	gSelectedProfile = gLoggedInProfile;
  	gPromptService = Components.classes[PromptService].getService(Components.interfaces.nsIPromptService);
  			
		document.getElementById("psAdminView").addEventListener("profile-settings-changed", profileSettingsChanged, false); 
		document.getElementById("psNonAdminView").addEventListener("profile-settings-changed", profileSettingsChanged, false); 
		document.getElementById("psAdminView").addEventListener("admin-privs-action", adminPrivsChange, false); 
		
  	gProfileSettingsView = getProfileSettingsView();
  	
  	//Register to call back custom Ok method
  	window.top.hPrefWindow.registerOKCallbackFunc(doOK);		
  	window.top.hPrefWindow.registerCancelCallbackFunc(doCancel);
	
		// Register data update callback.
		window.top.hPrefWindow.registerDataUpdateFunc("catProfilesButton", dataUpdateHandler);
																														  	
		refreshProfileSelector();      	 	
		refreshAutoLoginList();	
    
   	// My profile is default item selected
   	profileUtils.updateZipCodeFromInstall(gSelectedProfile);  	
  	gProfileSettingsView.profile = gLoggedInProfile;   
  	
  	bindForm();
  	doLayout();
  	return;

  }
  catch(e) {
    window.close();
    throw (e);
  }
}

function adminPrivsChange(){
	
	debug('adminPrivsChange');
		
	var adminProfiles = profileUtils.getAdminProfiles();     	
	var retVal = new Object();
	retVal.val = 1;
	
	// We need to apply some additional logic when the 
	// current user attempts to remove admin privs from his
	// or her own account
	if(profileUtils.getLoggedInProfile() == gSelectedProfile && !gProfileSettingsView.isAdministrator){
		
		debug('user is trying to remove privileges from his or her own account');
		var parm = new Array();         	
   	parm[0] = "Remove Administrative Privileges";
   	parm[1] = "Cancel"
   	parm[2] = "Remove Privileges"
   	var desc = new Array();
   	desc[0] = "By removing your administrative privileges, you will " +
							"no longer be able to change other profiles. This can "+
							"only be undone by another administrative user.";
		desc[1] = "Are you sure you want to remove your privileges?";
		
    openDialog("chrome://browser/content/pref/pref-custom-dialog.xul","", "chrome,close=no,dependent", parm, desc, retVal);
				                         		        
		if(retVal.val == 1){
	 		
			debug("adminCheckBoxClicked: we asked to proceed " + gProfileSettingsView.isAdministrator)
														
			// Ensure that there is at least one other admin profile
			// This should never happen as the profileSettings binding 
			// should disable this field if this user is the lone 
			// admin
   		var count = adminProfiles.length - 1;   		
   		if(count == 0){
        var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
	                                .getService(Components.interfaces.nsIPromptService);
       	promptService.alert(window,
			                          "Profile Manager Error",
			                          "Unable to remove Admin Privileges. At least one Profile must be Admin");
			  gProfileSettingsView.isAdministrator = true;	
			}				  					  				          					  	       
	    	
		}else {
			// The user has elected to not proceed with removing privs
			gProfileSettingsView.isAdministrator = true;
	  }	
	}	
			
	// If the user removes admin privs from his or her account and
	// this user is set to be the default user then turn off the 
	// auto login feature
	if (!gProfileSettingsView.isAdministrator && 
			gAutoLoginMenulist.value == gSelectedProfile.name) {							
			debug('remove autologin feature');                          
			gUserLoginRadioGroup.value = "NoAutoLogin";			
			upDateDefaultUserLogin();                                                   								
	}			
	profileSettingsChanged();			
}

function getDatasourceFromTree(tree){
	debug('getDatasourceFromTree - start - ' + tree.id);
	// Clear out the existing datasources          
	var datasources = tree.database.GetDataSources();
	while (datasources.hasMoreElements()) {       
		return datasources.getNext().QueryInterface(Components.interfaces.nsIRDFDataSource);
	}    		
	debug('getDatasourceFromTree - end');
}

function refreshTree(tree, ds, ref){
	debug('refreshTree - start');
	// Clear out the existing datasources          
	var datasources = tree.database.GetDataSources();
	while (datasources.hasMoreElements()) {       
		tree.database.RemoveDataSource(datasources.getNext());
	}    
	
	if (ref) tree.ref = ref;
	tree.database.AddDataSource(ds);
	tree.builder.rebuild();	
	debug('refreshTree - end');
}

function modeChanged() {	
	debug('modeChanged()');	
	profileUtils.profileListLocal[gSelectedProfile.name].setParentalControlMode(getParentalControlMode());	
	saveForm();
	bindPrntlCtrlLists(true);
	doLayout();
	
}

function getProfileSettingsView(){
		
	if (gLoggedInProfile.isAdmin == true) {  
		return document.getElementById("psAdminView");  		  	
	} else {
		if (profileUtils.profileListLocal[gSelectedProfile.name].createdThisSession){
			return document.getElementById("psAdminView");  
		} else {
			return document.getElementById("psNonAdminView");  		
		}
	}
}

function bindPrntlCtrlLists(doRequery){
										
		debug('profile loaded - refreshing lists');	
		var ds = profileUtils.profileListLocal[gSelectedProfile.name].getParentalControlLists(doRequery);
				
		var treeProfileSiteList = document.getElementById("treeProfileSiteList");
		refreshTree(treeProfileSiteList, ds, 'http://home.netscape.com/PC-rdf/MasterList');

		var treeGlobalAllowList = document.getElementById("treeGlobalAllowList");
		refreshTree(treeGlobalAllowList, ds, 'http://home.netscape.com/PC-rdf/GlobalAllowList');
		
		var treeGlobalBlockList = document.getElementById("treeGlobalBlockList");
		refreshTree(treeGlobalBlockList, ds, 'http://home.netscape.com/PC-rdf/GlobalBlockList');
		
		debug('bindPrntlCtrlLists - end');	
		
		//show_rdf_ds(ds);
}




function onProfileNameBroadCasterAttribChanged(){	
	gSelectorView.selected.bind();	
	doLayout();
}

function addSiteOrProviderToList(aScope, aListType){
	
	debug('addSiteOrProviderToList(' + aScope + ',' + aListType + ')');
	
	try {
		openDialog("chrome://browser/content/pref/pref-parental-ctrls-add-remove-site-settings.xul",
				"", "chrome,modal", profileUtils.profileListLocal[gSelectedProfile.name], aScope, aListType, profileUtils);
				
	} catch (e) {
		
		debug('addSiteOrProviderToList failed - ' + e.message);
		
	}
	
}

function doRemove(siteParam){
	debug('doRemove() - start');		
				
	try {							
		// Resource is a site
		if (siteParam.type == 'URISpec'){ 					
			if (siteParam.listType == 'UserList'){ 
				profileUtils.profileListLocal[gSelectedProfile.name].removeSiteFromProfile(siteParam.siteName, siteParam.permitType == 'Allow') 				
				// Resource is global
			} else {
				profileUtils.profileListLocal[gSelectedProfile.name].removeSiteFromGlobal(siteParam.siteName, siteParam.permitType == 'Allow') 				
			}				
		// Resource is a provider
		} else {

			// Resource is specific to a profile
			if (siteParam.listType == 'UserList'){ 
				profileUtils.profileListLocal[gSelectedProfile.name].removeListFromProfile(siteParam.providerID, siteParam.permitType == 'Allow') 
				// Resource is global
			} else {
				profileUtils.profileListLocal[gSelectedProfile.name].removeListFromGlobal(siteParam.providerID, siteParam.permitType == 'Allow') 
			}		
		}	
	} catch (e) {
		debug('doRemove error: ' + e.message);
		throw e;
	}		
	debug('doRemove() - end');			
}

function siteParams() {		
	
	this.providerID = null;																		
	this.siteName = null;																		
	this.permitType = null;				
	this.listType = null;
	this.type = null;
				
}

siteParams.prototype = {
		
		dump : function() {
			
			dump('========siteParams===========\n');																		
			dump('providerID = ' +  this.providerID+'\n');																		
			dump('siteName = ' +  this.siteName+'\n');																		
			dump('permitType = ' +  this.permitType+'\n');																		
			dump('listType = ' +  this.listType+'\n');																		
			dump('type = ' +  this.type+'\n\n');				
		}
};
				
function removeSiteOrProviderFromList(aScope) {
	debug('removeSiteOrProviderFromList() - start');		
	var gTreeProfileSiteList = document.getElementById("treeProfileSiteList");
	var gTreeGlobalBlockList = document.getElementById("treeGlobalBlockList");
	var gTreeGlobalAllowList = document.getElementById("treeGlobalAllowList");			
	var paramasArray = new Array();
	try {
		// Figure out which tree and datasource to use
		if (aScope == 'GLOBAL') {					
			if (gTreeGlobalAllowList.currentIndex > -1) {
				var ds = getDatasourceFromTree(gTreeGlobalAllowList)
				var gTargetTree = gTreeGlobalAllowList;			
			} else if (gTreeGlobalBlockList.currentIndex > -1){
				var ds = getDatasourceFromTree(gTreeGlobalBlockList); 	
				var gTargetTree = gTreeGlobalBlockList;						
			} else {
				// Do nothing if neither tree has a selection
				debug('Do nothing as no site selected');	
				return;
			}
			
		} else {		
				var ds = getDatasourceFromTree(gTreeProfileSiteList); 													
				var gTargetTree = gTreeProfileSiteList;			
		}
		
		var start = new Object();
		var end = new Object();
		var numRanges = gTargetTree.view.selection.getRangeCount();
		
		// Iterate through the selections and build up an array of siteParams objects.
		// The reason we build up a parameter list first and then make calls
		// to the nsParentalControlsService.removeSite/List is that these functions recreate 
		// the datasource so any reference to a specific rdf resource in the 'orginal' rdf
		// will not exist in the new datasource 
		for (var t=0; t<numRanges; t++){
		  gTargetTree.view.selection.getRangeAt(t,start,end);
		  for (var v=start.value; v<=end.value; v++){
		  			  	
	  		var res = gTargetTree.builder.getResourceAtIndex(v);	
	  		var resListType = ds.GetTarget(res, gPC_LIST_TYPE,true).QueryInterface(Components.interfaces.nsIRDFLiteral).Value;														
				var resType = ds.GetTarget(res, gPC_TYPE,true).QueryInterface(Components.interfaces.nsIRDFLiteral).Value;													
				var resPermitType = ds.GetTarget(res, gPC_PERMIT_TYPE,true).QueryInterface(Components.interfaces.nsIRDFLiteral).Value;															

	  		var parms = new siteParams();
	  		
	  		if (resType == 'URISpec'){	  			
	  			parms.siteName = new String(ds.GetTarget(res, gPC_NAME,true).QueryInterface(Components.interfaces.nsIRDFLiteral).Value);																		
	  			debug(parms.siteName);
	  		} else {
	  			parms.providerID = new String(ds.GetTarget(res, gPC_PROVIDER_ID,true).QueryInterface(Components.interfaces.nsIRDFLiteral).Value);																				  			
	  			debug(parms.providerID);
	  		}
	  		
				parms.permitType = new String(resPermitType);				
				parms.listType = new String(resListType);
				parms.type = new String(resType);
	  		paramasArray.push(parms);
		  				  				  						
		  }		  		  		  
		}		
						
		for (i in paramasArray){					
				//paramasArray[i].dump();		
				doRemove(paramasArray[i]);								
		}		  												
				 		
		//bindPrntlCtrlLists();			
		
	} catch (e) {
		debug('removeSiteOrProviderFromList error: ' + e.message);
	}	
	debug('removeSiteOrProviderFromList() - end');		
}


function autoLoginSelected(){
	
	debug('autoLoginSelected()');					
	saveForm();	
	bindForm();
	doLayout();	
}


function refreshProfileSelector() {
	dump("**** refreshProfileSelector()\n");
	while (gBoxOtherProfilesContainer.lastChild){
		gBoxOtherProfilesContainer.removeChild(gBoxOtherProfilesContainer.lastChild);
	}

	try {

		var profileList = profileUtils.getProfiles();

		debug('building profile list - start');
		// Build up the profile selection list
		while (profileList.hasMoreElements()) {
			var profile = profileList.getNext().QueryInterface(Components.interfaces.nsIToolkitProfile);			

			if (profile === gLoggedInProfile) {
				gMyProfile.profile = profile;
			} else {
				var item = document.getElementById("cloneThis").cloneNode(true);
				item.profile = profile;
				item.hidden = false;
				item.setAttribute("class", "viewListItem");
				gBoxOtherProfilesContainer.appendChild(item);
			}
		}
		
		setSelectedProfile();

		debug('building profile list - end');
	} catch (e) {
		debug('refreshProfileSelector() - failed'); 
		throw e;
	}
}
// Populate the autologin listbox
// Only admins will appear in this list
// to prevent a situation where a non-admin is
// auto logged in thus preventing any further admin
// type activities with the browser.
function refreshAutoLoginList() {
	debug('refreshAutoLoginList - start');
	
	try {
		
		while (gAutoLoginMenulist.firstChild.lastChild){
			gAutoLoginMenulist.firstChild.removeChild(gAutoLoginMenulist.firstChild.lastChild);
		}
	
		var profileList = profileUtils.getProfiles();		
		while (profileList.hasMoreElements()) {
				var profile = profileList.getNext().QueryInterface(Components.interfaces.nsIToolkitProfile);
				if(profile.isAdmin){											
					var menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", profile.displayName );
					menuitem.setAttribute("value", profile.name );
					gAutoLoginMenulist.firstChild.appendChild(menuitem);												
				}				
		}
			
	} catch (e) {
		debug('refreshAutoLoginList() - failed' + e.message); 
		throw e;
	}
	debug('refreshAutoLoginList - end');
}


function loadPictureDialog() {
		
	openDialog("chrome://browser/content/pref/pref-add-profile-picture.xul",
			"", "chrome,modal", gSelectedProfile);
				
	gProfileSettingsView.bind();
	refreshProfileSelector();	
}

	
function doOK()
{	
	// If a panel other than Profiles is showing when OKing.
	if (parent.profileData.isDataValid)	
	{
		parent.profileData.profileUtils.flush();
	}
	else
	{		
		var loggedInProfile = profileUtils.getLoggedInProfile();
		 	
		if (gProfileSettingsView.validate()){
			saveForm();
			profileUtils.flush();					
		}else 
		{
			return false;	
		}
	}
}

// This function is called when navigating away from the 
// Profiles panel. Populates the parent object with data 
// needed when the OK or Cancel button is pressed.
function dataUpdateHandler(){
	parent.profileData.profileUtils = profileUtils;
	parent.profileData.isDataValid = true;
	if(gProfileSettingsView.validate()){
		saveForm();		
		return true;
	}else {
		return false;
	}
}

function removeProfileClicked() {
	var selectedProfile = getSelectedProfile();
	
	var idx = -1;
	// find the index of the selected profile in boxOtherProfilesContainer
	for(var i = 0; i < gBoxOtherProfilesContainer.childNodes.length; i++)
	{
		if(gBoxOtherProfilesContainer.childNodes[i].profile === selectedProfile)
		{
			idx = i;
			break;
		}
	}
	
	var prevIdx = 0;
	if(idx > 0)
	{
		prevIdx = idx - 1;
	}
	
	profileUtils.removeProfile(selectedProfile);
	//arrange gBoxOtherProfilesContainer to show the available profiles (not logged in)
	refreshProfileSelector();
	
	refreshAutoLoginList();					
	// if there are more than 1 profile left
	if(gBoxOtherProfilesContainer.childNodes.length)
	{		
		// select the prev profile in the list
		gSelectedProfile = gBoxOtherProfilesContainer.childNodes[prevIdx].profile;	
	}
	else
	{
		// select logged in profile
		gSelectedProfile = gLoggedInProfile;
	}
	
	setSelectedProfile();
	// show the selected profile in the view
	bindForm();
	doLayout();
}


function doCancel(){
	var profileUtilities;

	// If a panel other than Profiles is showing when cancelling.
	if (parent.profileData.isDataValid){
		profileUtilities = parent.profileData.profileUtils;
	}	else {
		profileUtilities = profileUtils;
	}

	if (profileUtilities) {	
		for (var name in profileUtilities.profileListLocal){			
			//delete directories of the profile that was created in this session
			if (profileUtilities.profileListLocal[name] && 
					profileUtilities.profileListLocal[name].createdThisSession){
				var obj = profileUtilities.profileListLocal[name];
				try {
					debug('Attempt to remove profile directories');
				  obj.profile.rootDir.remove(true);				  
				}
				catch(e) {
					debug('Failed to remove profile directories -' + e.message);
				}
			}
		}	
	}
	
	// This call ensures that all changes made to the in-memory profile
	// data is rolled back so that when the flush to in the ini file
	// occurs the correct data is written
	profileUtilities.PROFILE_SVC.refreshService();
			
}

function addProfileClicked() {

	debug('addProfileClicked()');	
	// validate before you allow add profile
	if(!gProfileSettingsView.validate())
		{
				return;	
		}
		


	try {
		// Save changes for the current profile
		saveForm();
						
		// Create a new profile
		var profile = profileUtils.createProfile();						
		profile.zipCode = profileUtils.getInstalledZipCode();
		// force password checkboxes if necessary
		if(profileUtils.getAllProfilesNeedPassword())		{
			profile.protectProfileWithPassword = true;
		}
		
		if(profileUtils.getAllProfilesNeedPasswordToLogin()){
			profile.protectProfileWithPassword = true;
			profile.requiresPasswordToLogin = true;
		}
						
		// Bind to the newly created profile						
		gSelectedProfile = profile;					
		refreshProfileSelector();		
		refreshAutoLoginList();	
		gProfileSettingsView.profile = gSelectedProfile;				
		bindForm();						
		doLayout();
	} catch (e) {
		debug('addProfileClicked - failed -' + e.message);
	}
	
	if( (profileUtils.getProfilesCount() == 2) &&
	    (gLoggedInProfile.isAdmin) &&
	    (profileUtils.getDefaultProfile()) )
	{
    const nsIPromptService = Components.interfaces.nsIPromptService;
    var promptSvc = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                              .getService(nsIPromptService);

    var flags = promptSvc.BUTTON_TITLE_IS_STRING * promptSvc.BUTTON_POS_0 +
                promptSvc.BUTTON_TITLE_IS_STRING * promptSvc.BUTTON_POS_1;


		var msg = "The Netscape Browser is currently set to automatically log " +
		          "in as " + gLoggedInProfile.name + " when the browser is started.\n\n" +
		          "Since there are now multiple browser profiles, would " +
		          "you like to disable the auto-login feature?";

  	var rv = promptSvc.confirmEx(window,
                                 "Turn Off Automatic Login?",
                                 msg,
                                 flags,
                                 "Keep Automatic Login", "Disable Automatic Login", null,
                                 null, {});
		
		// if Disable clicked, turn off auto-login
		if(rv == 1)
		{
			debug("Disable auto-login\n");
			profileUtils.setDefaultProfile("");
			profileUtils.setStartWithLastProfile(false);
			gUserLoginRadioGroup.value = "NoAutoLogin";
		  gUserLoginRadioGroup.selectedIndex = 0;
		  gAutoLoginMenulist.disabled = true;
		}
	}
	
	gProfileSettingsView.setFocusToDisplayNameField();
	
}


function setSelectedProfile(){
	debug('setSelectedProfile()');		 	 
	gSelectorView.removeEventListener("richview-select", viewListItemSelected, false);
	gSelectorView.removeEventListener("richview-pre-select", viewListItemPreSelected, false);
	try {
		// Get a reference to all the profile selector items
		var selectorItems = document.getElementsByAttribute('class', 'viewListItem');	
		if (gSelectedProfile){		
			for (var i = 0; i < selectorItems.length; ++i) {			
				if (selectorItems[i].profile === gSelectedProfile) {
					debug('about to set selected property');		
					gSelectorView.selected = selectorItems[i];														
				}		
			}
		} else if (selectorItems.length >= 0) {			
			gSelectorView.selected = selectorItems[0];
		}		
  
	} catch (e) {
		debug('setSelectedProfile - failed -' + e.message);
	}
	gSelectorView.addEventListener("richview-select", viewListItemSelected, false);
	gSelectorView.addEventListener("richview-pre-select", viewListItemPreSelected, false);
}


function upDateDefaultUserLogin(){		
	debug('upDateDefaultUserLogin()');			
	saveForm();	
	bindForm();
	doLayout();
}

function updateShowProfileCreationButtonCheck(){		
	debug('updateShowProfileCreationButtonCheck()');			
	saveForm();	
	bindForm();
	doLayout();
}

function updateAllowNewProfilesByAnyone(){	
	debug('updateAllowNewProfilesByAnyone()');				
	saveForm();	
	bindForm();
	doLayout();
}

function updateAllProfilesNeedPassword(){
	debug('updateAllProfilesNeedPassword()');				
	saveForm();	
	bindForm();
	doLayout();
}

function updateAllProfilesNeedPasswordToLogin(){
	debug('updateAllProfilesNeedPasswordToLogin()');				
	saveForm();	
	bindForm();
	doLayout();
}
