var _elementIDs = ["rgShowWarning","cbScanDownload","rgAutoDelete","cbDeleteNotify","mlBlacklistUpdateFreq"];

// Preference tags
var kprefSpywareIsInstalled = "spywatch.isInstalled";
var kprefSpywareShowWarning = "spywatch.showWarning";
var kprefSpywareScanDownloads = "spywatch.scanDownloads";
var kprefSpywareAutoDelete = "spywatch.autoDeleteSpyware";
var kprefSpywareNotifyDeleted = "spywatch.notifySpywareDeleted";
var kprefSpywareBlacklistUpdate = "spywatch.blacklistUpdateFreq";
var kprefSpywareLastScan = "spywatch.lastFullScan";
var kprefSpywareIsScanningFile = "spywatch.scanning.file";
var kprefSpywareIsScanningSystem = "spywatch.scanning.system";

// time constants for last system scan performed
var ktime59minutes = 3540;  // 59 minutes, 30 secs
var ktime01hours = 3600;
var ktime23hours = 82800;
var ktime01days = 86400;
var ktime06days = 518400;
var ktime01weeks = 604800;
var ktime04weeks = 2419200;
var ktimehalfhour = 1800;
var ktimehalfday = 43200;
var ktimehalfweek = 302400;

var gSpywareBundle = 0;
var gPrefService = 0;

function Startup() {
  gSpywareBundle = document.getElementById('bundle_spywatch');
	gPrefService = Components.classes["@mozilla.org/preferences-service;1"]
	                         .getService(Components.interfaces.nsIPrefBranch);

  spywatchStatusUpdate();

	dump('Spyware Startup()\n');
}

function showFullScan() {
	var promptSvc = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
	if (!spyWareScanner.isInstalled()) {
		var msg = "A serious error has occurred, please reinstall Netscape.";
		promptSvc.alert(null,"Alert", msg);
	} else {
		try
		{
		spyWareScanner.performFullScan();
	}
	catch (e)
	{
		var msg = gSpywareBundle.getString('fullscanerrorcaption');
		promptSvc.alert(null,"Alert", msg);
	}
	}
}

function isScanAvailable(target) {
  if (target instanceof Components.interfaces.nsIDOMXULCheckboxElement) {
    if (target.checked) {
      if (!spyWareScanner.isInstalled()) {
        target.checked = false;
      }
    }
  }
  
  // change the enabled state of dependent controls
  updateEnabledState(target);
}

function updateEnabledState(target) {
  if (!target) {
    target = document.getElementById('cbScanDownload');
  }

  var radiogrp = document.getElementById('rgAutoDelete');
  var chkbox = document.getElementById('cbDeleteNotify');

  radiogrp.disabled = !target.checked;
  if (!target.checked) {
    chkbox.disabled = true;
  } else {
    chkbox.disabled = ('true' != radiogrp.value);
  }
}

function spywatchStatusUpdate() {
  // set status label
  var label = document.getElementById('spywatchStatusLabel');
  if ( !gPrefService.getBoolPref(kprefSpywareScanDownloads) ) {
    label.value = gSpywareBundle.getString('disabled');
  }

  else {
    var strStatus = gSpywareBundle.getString('enabled');

    if ( gPrefService.getBoolPref(kprefSpywareIsScanningFile) ) {
      strStatus += ' ' + gSpywareBundle.getString('scanfile');
    }
    else if ( gPrefService.getBoolPref(kprefSpywareIsScanningSystem) ) {
      strStatus += ' ' + gSpywareBundle.getString('scansystem');
    }
    else {
      strStatus += ' ' + gSpywareBundle.getString('scanstandby');
    }

    label.value = strStatus;
  }

  // set time of last full system scan
  label = document.getElementById('spywatchLastScanLabel');
  var lastScan = gPrefService.getIntPref(kprefSpywareLastScan);

  if (0 == lastScan) {
    // indicates a scan has never happened
    label.value = gSpywareBundle.getString('lastscannever');
    return;
  }

  // Determine which string to use based on last scan duration
  var lastDate = new Date(lastScan * 1000);
  var now = new Date();
  dump('Now: ' + now.valueOf() + '\n');
  var duration = Math.round((now.valueOf() - lastDate.valueOf()) / 1000);   // keep as seconds
  dump('Duration: ' + duration + '\n');
  var strLastScan;
  var nTimeSpan = 0;

  if (duration < ktime59minutes) {
    // measured as less than an hour
    strLastScan = gSpywareBundle.getString('lastscanlessthanhour');
  }

  else if ( duration < (ktime23hours + ktimehalfhour) ) {
    // measured in hours - determine amount
    nTimeSpan = Math.floor(duration / ktime01hours)
    if ( 1 == nTimeSpan ) {
      strLastScan = gSpywareBundle.getString('lastscanonehour');
    } else {
      strLastScan = gSpywareBundle.getString('lastscansomehours');
    }
  }

  else if ( duration < (ktime06days + ktimehalfday) ) {
    // measured in days - determine amount
    nTimeSpan = Math.floor(duration / ktime01days)
    if ( 1 == nTimeSpan ) {
      strLastScan = gSpywareBundle.getString('lastscanoneday');
    } else {
      strLastScan = gSpywareBundle.getString('lastscansomedays');
    }
  }

  else if ( duration < (ktime04weeks + ktimehalfweek) ) {
    // measured in weeks - determine amount
    nTimeSpan = Math.floor(duration / ktime01weeks)
    if ( 1 == nTimeSpan ) {
      strLastScan = gSpywareBundle.getString('lastscanoneweek');
    } else {
      strLastScan = gSpywareBundle.getString('lastscansomeweeks');
    }
  }

  else {
    // show date
    strLastScan = lastDate.toLocaleDateString();
  }

  if ( nTimeSpan > 1 ) {
    strLastScan = strLastScan.replace('%1', '' + nTimeSpan)
  }

  label.value = strLastScan;
}

function ClearOverrideHash(aTarget) {
	if (aTarget.value == "true" && sitecontrols.SCSVC) {
		sitecontrols.SCSVC.clearOverrideHash();
	}
}