var dashboard = {

	/*declare each panel as a member of the dashboard object this will faciliate easier state
	  management of the separate dashboard component panels */
	mObserver: null,
	mAutoExposureTimer: null,
	mCurrentSiteStatus: null,
	mCurrentBrowserLevel: null,
	mCurrentSiteWarningType: null,
	mPopupBlocking: null,
	mMasterPopupVerified: null,
	mMasterPopupUnverified: null,
	mMasterPopupNotTrusted: null,
	mIdentityGuard: null,
	mLastSilentUpdate: null,
	mLastAppUpdate: null,
	mSpywatchIsInstalled: null,
	mSpyWatchActiveScanOn: null,
	mYellowThreatsArray: null,
	mRedThreatsArray: null,
	mDashboardIDThreatActiveScan: null,
	mBlackListUpdateTooLong: null,
	mBlackListDownloadError: null,
	mTimer: null,
	mNumPanels: 5,
	mUpdateAvailable: null,
	mStatusBarPanel: null,
	mAutoExposeProblem: null,
	mAutoExposeDaily: null,
	THREAT_OPEN_DASHBOARD_INTERVAL: 8000,
	mPanels : null,
	mSpyWareLastFullScan: null,
	mOpacityIncrement: 0.045,
	kSpyware: 0,
	kPhishing: 1,
	kUnverifiedSite: 2,
	kBlackListUpdateError: 3,
	kPopupBlockingDisabled: 4,
	kCriticalUpdate: 5,
	kFullScanNotYetRun: 6,
	kActiveXOn: 7,
	kIDTheftUpdateDelayTooLong: 8,
	kSpywareUpdateDelayTooLong: 9,
	mSilentUpdateInterval: 0,
	k_DASHBOARD_INVOCATION_TYPE_NORMAL: 0,
	k_DASHBOARD_INVOCATION_TYPE_DAILY_AUTOEXPOSURE: 1,
	k_DASHBOARD_INVOCATION_TYPE_PROBLEM_AUTOEXPOSURE: 2,
	ONE_DAY_IN_SECONDS: 86400,
	mCurrentSiteHost: null,
	mSiteControl: null,
	mEngine: null,
	mAllowImages: null,
	mEnableActiveX: null,
	mEnableJava: null,
	mEnableJavaScript: null,
	mAllowCookies: null,
	mAutoExposureWarnings: Array(4),
	mNewestBrowserVersion: null,
	mWindowWatcher : null,
	mDashboardBundle : null,
	mDashboardWindow: null,
	mShouldExpose: null,
	mDailyExposureCloseTimer: null,
	mDashboardWindowLoaded: false, 

	/**********************************************************/
	/* init()
	/* Initialized the dashboard with the current state of
	/* of any security threats. Any time a threat it is discovered
	/* it is stored in the member variable mYellowThreatsArray pr
	/* mRedThreatsArray.
	/**********************************************************/
	init: function() {
		// get the status flags for the various security center panels
		
		this.mObserver = Components.classes["@mozilla.org/observer-service;1"]
         .getService(Components.interfaces.nsIObserverService);
         
    if (!gPrefService.getPrefType('browser.securitycenter.autoexposure_interval')) {
    	this.ONE_DAY_IN_SECONDS = 86400;
    	gPrefService.setIntPref('browser.securitycenter.autoexposure_interval', this.ONE_DAY_IN_SECONDS)
    }
    
    this.mAutoExposeDaily = gPrefService.getBoolPref('browser.securitycenter.showdailyautoexposure');
    this.ONE_DAY_IN_SECONDS = gPrefService.getIntPref('browser.securitycenter.autoexposure_interval');
		var sc = sitecontrols.SCSVC;
		
 		this.mWindowWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                        .getService( Components.interfaces.nsIWindowWatcher );

		// initialize the local threat arrays, these are used to track the threats occurred
		// during initialization of this instance of the security dashboard
		this.mYellowThreatsArray = Array();
		this.mRedThreatsArray = Array();
		this.mStatusBarPanel =document.getElementById('securityDashboard');

		try {
			this.mSpyWatchActiveScanOn = gPrefService.getBoolPref('spywatch.scanDownloads');
		} catch(ex) {
			this.mSpyWatchActiveScanOn = false;
		}
		
		try {
			this.mSpywatchIsInstalled = gPrefService.getBoolPref('spywatch.isInstalled');
		} catch (ex) {
		}
		
		try {
			this.mDashboardBundle = document.getElementById('dashboardBundle');
		} catch (ex) {
			this.debug(ex);
		}
		
		// set autoexposure prefs to the UI
		this.mAutoExposeProblem = gPrefService.getBoolPref('browser.securitycenter.autoexpose');
		
		try {
			this.mDashboardIDThreatActiveScan = gPrefService.getBoolPref('identityTheft.ActiveScanning');
		} catch (ex) {

		}

		// HAXOR -- on init there are no updates available until this value is actually changed
		this.mUpdateAvailable = -1

		if (sc) {
			// set the dashboard's site ratings data members
			this.SetSiteRatings(sc);

			// set the  dashboard site settings data members
			this.SetSiteSettings(sc);

			// set the popup blocking settings
			this.SetPopupBlockingSettings(sc);


		} else {
				//this.debug("Critical ERROR: can't get handle on site controls!!");
		}

		// set the silent update time
		this.SetSilentUpdateTime();

		// set the last app update time
		this.SetLastAppUpdateTime();

		// set the last spyware scan
		this.SetLastSpywareScan();

		this.CheckBrowserUpdatePrefs();

		// push any warnings found in init() to teh the the red and yellow arrays
		this.PushWarnings();
		
		this.NotifyDashboardObserver();

		// no threats so reset the statusbar region back to its default state
		if (threatsQueue.mNumThreats == 0) {
				this.resetStatusbarPanelStatus();
			}
		
		// the dashboard is open, update the UI now
		if (dashboard.mDashboardWindow != null) {
			if (dashboard.mDashboardWindowLoaded)
				dashboard.mDashboardWindow.UpdateSecurityDashboard();
		} 
	},

	/**********************************************************
	 * SetNewestVersion
	 * @param: none
	 * @returns: nothing;
	 * Description: 
	 *  Uses the Update Service to set the newest browser 
	 *  version.
	 */
	SetNewestBrowserVersion: function() {
		// BC: get the latest version of the browser
	  var updatesvc = Components.classes["@mozilla.org/updates/update-service;1"]
	                            .getService(Components.interfaces.nsIUpdateService);
	  var _newestInfo = updatesvc.newestVersion;
	  if (_newestInfo) {
	    this.mNewestBrowserVersion = _newestInfo.updateDisplayVersion;
	  }	
	},
	/**********************************************************/
	/* SetSilentUpdateTime()
	/* Sets the last time (in minutes) that a silent update
	/* was performed.
	/**********************************************************/
	SetSilentUpdateTime: function() {
		// update the last silent update time
		var lastSilentUpdateEpoch = gPrefService.getIntPref('silent.update.lastUpdateDate');
		var diffEpoch = this.GetNowInSeconds() - lastSilentUpdateEpoch;

		// last silent update in minutes
		this.mLastSilentUpdate = Math.floor((diffEpoch / 60));	
		this.mSilentUpdateInterval = gPrefService.getIntPref('silent.update.interval')/60000;
		
		if (this.mLastSilentUpdate > this.mSilentUpdateInterval) {
			this.mBlackListUpdateTooLong = true;
		} else {
			this.mBlackListUpdateTooLong = false;
		}
	},

	InitAutoExposedWarnings: function() {
		if (this.mBlackListDownloadError == true) {
			this.mAutoExposureWarnings['sdIdentityGuard'] = "Unable to Update Blacklist";
			this.mAutoExposureWarnings['sdSpyWatch'] = "Unable to Update Blacklist";
		} 
				
		if (this.mCurrentSiteWarningType == "spyware") {
			this.mAutoExposureWarnings['sdSpyWatch'] = "Suspected Spyware Site";
			this.mAutoExposureWarnings['sdSpyWatch'] = "Suspected Spyware Site";
		}
		
		if (this.mCurrentSiteWarningType == "phishing") {
			this.mAutoExposureWarnings['currentSiteStatus'] = "Suspected Identity Theft Site";
			this.mAutoExposureWarnings['sdIdentityGuard'] = "Suspected Identity Theft Site";
		} else {
			this.mAutoExposureWarnings['currentSiteStatus'] = "Suspected Spyware Site";
		}
			
		this.mAutoExposureWarnings['sdBrowserUpdates'] = "Browser Available";
	},
	
	/**********************************************************/
	/* SetLastAppUpdateTime()
	/* Sets the last time (in minutes) that the browser was
	/* updated.
	/**********************************************************/
	SetLastAppUpdateTime: function() {
		// update the last brower update time
		var lastAppUpdate = gPrefService.getIntPref('app.update.lastUpdateDate');
		if (lastAppUpdate > 0)
			this.mLastAppUpdate = (this.GetNowInSeconds() - gPrefService.getIntPref('app.update.lastUpdateDate'));
		else 
			this.mLastAppUpdate = -1;
	},

	/**********************************************************/
	/* SetSiteSettings()
	/* Sets the last time (in minutes) that the browser was
	/* updated.
	/**********************************************************/
	SetSiteSettings: function(sc) {
		if (sc) {
			// get the site control settings for particular properties
			var currentURI = gBrowser.currentURI;
			var site = sc.getResourceForURI(currentURI.spec);
			var properties = ['displayEngine', 'allowImages', 'enableActiveX',
		                    'enableJava', 'enableJavaScript','allowCookies'];
		   for (var p in properties) {
				try {
					var value = sc.readSiteControlResource(site, properties[p]);

					switch (properties[p]) {
						case "displayEngine":
							if (value === "Gecko") {
								dashboard.mEngine = "Firefox";
								// this should automatically be switched in the sitecontrols service..
								dashboard.mEnableActiveX = "Disabled";
							}
							else
								dashboard.mEngine = "Internet Explorer";
						break;

						case "allowImages":
							if (value == 'true')
								dashboard.mAllowImages = "Allowed";
							else
								dashboard.mAllowImages = "Disallowed";
						break;

						case "enableActiveX":
							if (value== 'true')
								dashboard.mEnableActiveX = "Enabled";
							else
								dashboard.mEnableActiveX = "Disabled";
						break;

						case "enableJava":
							if (value== 'true')
								dashboard.mEnableJava = "Enabled";
							else
								dashboard.mEnableJava = "Disabled";
						break;

						case "enableJavaScript":
							if (value== 'true')
								dashboard.mEnableJavaScript = "Enabled";
							else
								dashboard.mEnableJavaScript = "Disabled";
						break;

						case "allowCookies":
							if (value=='true')
								dashboard.mAllowCookies = "Enabled";
							else
								dashboard.mAllowCookies = "Disabled";
						break;

					}

				} catch (ex) {
					dashboard.debug('WARNING: error copying property "'+prop+'"');
				}
			}
		}
	},

	/****************************************************************/
	/* SetLastSpywareScan()
	/* Sets the dashboard with the last SpyWare Scan
	/*****************************************************************/
	SetLastSpywareScan: function() {
		// get the spyware last full scan date from pref service
		this.mSpyWareLastFullScan = 0;
		try {
			this.mSpyWareLastFullScan = gPrefService.getIntPref('spywatch.lastFullScan');
			this.debug(this.mSpyWareLastFullScan);
		} catch (e) {
			this.debug(e);
			this.mSpyWareLastFullScan = 0;
		}
	},

	/****************************************************************/
	/* SetPopupBlockingSettings()
	/* Sets the dashboard with the browser's popup blocking settings
	/*****************************************************************/
	SetPopupBlockingSettings: function(sc) {
		// set the member variable for popupblocking
		var currentURI = gBrowser.currentURI;
		var site = sc.getResourceForURI(currentURI.spec);

		if (sc.readSiteControlResource(site, 'allowPopups') == "true") {
			this.mPopupBlocking = false;
		} else {
			this.mPopupBlocking = true;
			// remove popup blocking disabled threat from the queue if it's there
			threatsQueue.remove(dashboard.kPopupBlockingDisabled);
		}

		// set master popup blocking settings	data members
		var verifiedDefaultSetting = sc.getResourceForURI("Verified Default");
		var unverifiedDefaultSetting = sc.getResourceForURI("Not Verified Default");
		var warningDefaultSetting = sc.getResourceForURI("Warning Default");

	 	this.mMasterPopupVerified = sc.readSiteControlResource(verifiedDefaultSetting, 'allowPopups');
	 	this.mMasterPopupUnverified = sc.readSiteControlResource(unverifiedDefaultSetting, 'allowPopups');
	 	this.mMasterPopupNotTrusted = sc.readSiteControlResource(warningDefaultSetting, 'allowPopups');
	},


	/****************************************************************/
	/* SetSiteRatings()
	/* Sets the dashboard member variables for the site settings
	/* of the currently loaded site in a tab
	/****************************************************************/
	SetSiteRatings: function (sc) {
			if (sc) {
				var currentURI = gBrowser.currentURI;
				var site = sc.getResourceForURI(currentURI.spec);
				var trustLevel;
				var securityLevel;
				
				if (currentURI.scheme == 'http' ||
				    currentURI.scheme == 'https' ||
				    currentURI.scheme == 'ftp')
				{
					trustLevel = sc.getSiteTrustLevel(currentURI.spec);
				} else if (currentURI.scheme == 'about') {
					/* When we navigate to a known spyware,phishing site the browser is redirected to about:blank then
					** about:*****?.... this causes a problem because now we don't know what site to check.
					** Luckly the url that was attempted to navigate to is in the about:*****?url=****** url :)
					*/
					
					// Check for spyware, phishing redirected sites (we can extract the url and check that with sitecontrols
					if (currentURI.spec.indexOf('about:spyware') == 0 ||
						  currentURI.spec.indexOf('about:phishing') == 0 ||
						  currentURI.spec.indexOf('about:phishing') == 0 ) {
							trustLevel = 0;
					} 
				} else if (!sc.isControllableURI(currentURI.spec)) {
					trustLevel = 0; // not controlled, so it's file:// or chrome://
				}
							
				// get partner information from site controls service
				var whiteListCode = sc.getSiteTrustLevelDetail(currentURI.spec, sc.SC_TRUST_WHITELIST_IDX);
				var whiteListStr = sc.getSiteTrustProviderName(whiteListCode);

				switch (trustLevel)
				{
					case -1:
						this.mCurrentSiteStatus ='disabled';
						this.mBrowserLevel = gPrefService.getCharPref("trustsettings.default");
						break;
						
					case 0:
						this.mCurrentSiteStatus = 'verified';
						this.mCurrentSiteWarningType = null;
						break;

					case 1:
						this.mCurrentSiteStatus = 'warning';
						this.mCurrentSiteWarningType = null;
						this.mYellowThreatsArray.push("unverified-site");
						break;
					case 2:
						
						this.mCurrentSiteStatus = 'critical';
						this.mCurrentSiteWarningType = 'phishing';
						break;

					case 4:
						this.mCurrentSiteStatus = 'critical';
						this.mCurrentSiteWarningType = 'spyware';
						break;

					case 6:
						this.mCurrentSiteStatus = 'critical';
						this.mCurrentSiteWarningType = 'spyware,phishing';
						break;

					default:
				}
				
				// check the browser settings for the active site
				//MERC-BC 
				//Add 'trustLevel != 0', if the trust lavel is 0(verified) 
				//then it doesn't need to be checked
				dashboard.debug(currentURI);
				if (currentURI.scheme != 'about') {
					try {
						if(trustLevel!=0&&!gPrefService.getBoolPref("trustsettings.partnerratings.auto")
								&& !sitecontrols.SCSVC.isControlledSite(currentURI.host)) {
							this.mBrowserLevel = gPrefService.getCharPref("trustsettings.default");
						} else {
							this.mBrowserLevel = sc.readSiteControlResource(site, 'securityLevel');	
						}
					} catch (ex) {
						dashboard.debug(ex);
					}
				}

		} else {
				//dump("Critical ERROR: can't get handle on site controls!!");
		}
	},



	/******************************************************************/
	/* CheckBrowserUpdatePrefs()
	/* Sets the threats if an extension or browser update is
	/* available and returns true
	/******************************************************************/
	CheckBrowserUpdatePrefs: function() {
		// determine if update(s) are available and set the memeber variable
		var updatesAvailable = gPrefService.getBoolPref('app.update.updatesAvailable');
		var lastAppLastUpdated = gPrefService.getIntPref('app.update.lastUpdateDate');
		var bCanUpdate = false;
		
		SEVERITY_HIGH = 2;
		SEVERITY_MEDIUM = 1;

		// if an update has been performed the update is not available anymore, we are
		// waiting for the user to restart the browser
		var updatePerformed = gPrefService.getBoolPref('app.update.performed');

		if (updatesAvailable && !updatePerformed) {
			var updateSeverity =  gPrefService.getIntPref('update.severity');
			dashboard.mUpdateAvailable = updateSeverity;
			// case: critical update ==> red threat
			if (updateSeverity == SEVERITY_HIGH) {
				threatsQueue.add(dashboard.kCriticalUpdate, "red");
			}
			// case: extension update ==> yellow threat
			else if (updateSeverity == SEVERITY_MEDIUM) {
				dashboard.mYellowThreatsArray.push("extension-update-available");
			}

			bCanUpdate = true;
		}
		
		return bCanUpdate;
	},
	
	
	/**********************************************************/
	/* GetNowInSeconds() 
	/* Helper function to get the current time in seconds
	/**********************************************************/
	GetNowInSeconds: function() {
		var d = new Date();
    return Math.floor(d.getTime()/1000);
	},

	/**********************************************************/
	/* GetNowInUTCSeconds() 
	/* Helper function to get the current time in UTC seconds
	/**********************************************************/
	GetNowInUTCSeconds: function() {
		var d = new Date();
    return Math.floor(Date.UTC(d.getUTCFullYear(),
                d.getUTCMonth(),
                d.getUTCDate(),
                d.getUTCHours(),
                d.getUTCMinutes(),
                d.getUTCSeconds(),
                d.getUTCMilliseconds())/1000);
	},

	/**********************************************************/
	/* NotifyDashboardObserver()
	/* Sends all Red and Yellow threats to teh dashboard
	/* Panel Observer so the UI can be updated
	/**********************************************************/
	NotifyDashboardObserver: function() {
		// notify statusbar security center of possible threats
		// red threats trump yellow ones..

		var activeBrowserWindow = document.getElementById('main-window');
		var windowID = 1;

		if (activeBrowserWindow) {
			windowID = activeBrowserWindow.getAttribute('windowID');
		}
		if (this.mRedThreatsArray.length > 0) {
			for (var i=0; i < this.mRedThreatsArray.length; i++) {
					if (this.mRedThreatsArray[i] == "critical-update-available") {
						this.mObserver.notifyObservers(null, this.mRedThreatsArray[i], this.mRedThreatsArray[i]);
					}
					else
						this.mObserver.notifyObservers(null, "site-security-alert-" + windowID, this.mRedThreatsArray[i]);
			}
		}
		var unverifiedSite = false;
		if (this.mYellowThreatsArray.length > 0) {
				for (var ii=0; ii < this.mYellowThreatsArray.length; ii++) {
					if (this.mYellowThreatsArray[ii] == "unverified-site") {
						this.mObserver.notifyObservers(null, "site-security-alert-" + windowID, this.mYellowThreatsArray[ii]);
						unverifiedSite = true;
					}

					if (this.mYellowThreatsArray[ii] == "site-security-alert" || this.mYellowThreatsArray[ii] == "popup-blocking-disabled") {
						this.mObserver.notifyObservers(null, "site-security-alert-" + windowID, this.mYellowThreatsArray[ii]);
					} else {
						this.mObserver.notifyObservers(null, this.mYellowThreatsArray[ii], null);
					}
				}
		}

		if (this.mRedThreatsArray.length == 0 && !unverifiedSite){
			// no alerts, send the message to the observer that every this is peachy
			this.mObserver.notifyObservers(null, "site-security-alert-" + windowID, "-1");
		}
	},


	/**********************************************************/
	/* makeDailyAutoExposureTimer()
	/* Creates a 1 day timer for the daily dashboard
	/* autonotification
	/**********************************************************/
	makeDailyAutoExposureTimer: function() {		
		var autoexposureInterval = gPrefService.getIntPref('browser.securitycenter.autoexposure_interval');	
		var lastAutoExposure = gPrefService.getIntPref('browser.securitycenter.last_daily_exposure');
		var now = this.GetNowInSeconds();
		var nextAutoExposureTime = now + autoexposureInterval;
		var timerDelay;

		// if the lastAutoExposure is 0 or the the next autoexposure time has been passed
		// show the dashboard
		if ((lastAutoExposure == 0 || now > nextAutoExposureTime) && this.mAutoExposeDaily) {
			gPrefService.setIntPref('browser.securitycenter.last_daily_exposure', now);
			this.debug('about to autoexpose');
			OpenDashboardWindow('fade');
			if (dashboard.mDashboardWindow) {
				setTimeout('dashboard.mDashboardWindow.ToggleDailyAutoexposeCheckboxVisibility()', 50);
			}
			// close after 8 seconds
			// MERC (rpaul) :: commenting this out for now, so the dashboard won't close after a daily autoexposure
			// again i think this is a timing issue.
			//this.mDailyExposureCloseTimer = setTimeout('CloseDashboardWindow()', 16000);
			timerDelay = this.ONE_DAY_IN_SECONDS;
		} else {
			// set the timer to be the difference between one hour from the last exposure time and now
			timerDelay = nextAutoExposureTime - now;
		}

		// timerdelay is in seconds, yet the timer runs in milliseconds so turn it into millseconds
		timerDelay = timerDelay*1000;

    if (!this.mTimer)
    	this.mTimer = Components.classes["@mozilla.org/timer;1"]
                              .createInstance(Components.interfaces.nsITimer);
    this.mTimer.cancel();
    this.mTimer.initWithCallback(this, timerDelay, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
	},


	/**********************************************************/
	/* Notify()
	/* Notification function for the daily autoexposure timer
	/**********************************************************/
	notify: function(aTimer) {
		// timer expired, set the pref for last daily exposure time, display the dashbboard
		// and restart the time
		gPrefService.setIntPref('browser.securitycenter.last_daily_exposure', this.GetNowInUTCSeconds());
		this.makeDailyAutoExposureTimer();
		OpenSecurityDashboard(dashboard.k_DASHBOARD_INVOCATION_TYPE_DAILY_AUTOEXPOSURE);
	},


	/**********************************************************/
	/* PushWarnings()
	/* Checks the dashboard warning member variables
	/* and pushes them to its threat array (red or yellow)
	/**********************************************************/
	PushWarnings: function() {
		// if there are warnings from the site from the security partner, push
		// them into the red threats array
		if (this.mCurrentSiteWarningType != null) {
			this.mRedThreatsArray.push(this.mCurrentSiteWarningType);
		}

		if (!this.mPopupBlocking) {
			this.mYellowThreatsArray.push("popup-blocking-disabled");
		}
		
		// display blacklist uneffective error if the blacklist can be downloaded
		if (this.mBlackListUpdateTooLong && !this.mBlackListDownloadError) {
			this.mYellowThreatsArray.push("spyware-uneffective");
			this.mYellowThreatsArray.push("idTheft-uneffective");
		} else {
			// this case isn't really needed, this is only for debugging
			// when the user messes with the system clock
			threatsQueue.remove(dashboard.kIDTheftUpdateDelayTooLong);
			threatsQueue.remove(dashboard.kSpywareUpdateDelayTooLong);
		}

		if (this.mSpyWareLastFullScan == 0) {
			dashboard.mYellowThreatsArray.push("full-scan-not-run");
		} else {
			// remove this threat from the queue if there was a scan run
			threatsQueue.remove(dashboard.kFullScanNotYetRun);
		}
	},
	
	GetAutoExposedWarning: function(aPanelId) {
		this.InitAutoExposedWarnings();
		if (aPanelId == "" || aPanelId == null)
			return;
		else 
			return this.mAutoExposureWarnings[aPanelId];
	},
	
	/*******************************************************************/
	/* flipToNextThreat(void)
	/* Finds the next threat in the threat queue and flips the
	/* stautusbar UI to it
	/*********************************************************************/
	flipToNextThreat : function() {
		if (threatsQueue.hasYellowThreats() || threatsQueue.hasRedThreats()) {
			threatsQueue.cycle();
		} else {
			var dashboardPanel = document.getElementById('securityDashboard');
			dashboardPanel.removeAttribute('warningLevel');
			var statusbarLabel = document.getElementById('securityDashboardLabel');
			statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardTitle'));
		}
	},
	
	/*******************************************************************/
	/* flipStatusbarPanelStatus(aThreatType)
	/* Changes the security dashboard statusbar panel to the 
	/* UI for the current threat								
	/*********************************************************************/
	flipStatusbarPanelStatus : function(aThreatType) {
		// get handle on the xul UI objects that need to be changed
		var dashboardPanel;
		var statusbarLabel;
		var statusbarThreatIcon;
		
		dashboardPanel = document.getElementById('securityDashboard');
		if (!dashboardPanel)
			dashboardPanel = DashboardUIController.statusbarPanel;
		
		statusbarLabel = document.getElementById('securityDashboardLabel');
		if (!statusbarLabel)
			statusbarLabel = DashboardUIController.statusbarLabel;
			
		statusbarThreatIcon = document.getElementById('currThreatItemIcon');
		if (!statusbarThreatIcon)
			statusbarThreatIcon = DashboardUIController.statusbarThreatIcon;
		
		var currentWarningLevel = dashboardPanel.getAttribute("warningLevel");
		
		switch (aThreatType) {
			case dashboard.kPhishing:
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatIDTheft'));
				// create the phishing indicator button
				statusbarThreatIcon.setAttribute("class", "identity-guard-icon");
			break;
			
			case dashboard.kSpyware:
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatSpyware'));
				statusbarThreatIcon.setAttribute("class", "status-spywatch-icon");
			break;
			
			case dashboard.kUnverifiedSite:
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatUnverifiedSite'));
				statusbarThreatIcon.setAttribute('browserSetting', this.mBrowserLevel);
				statusbarThreatIcon.setAttribute("class", "security-dashboard-current-rating");
			break;
		
			case dashboard.kBlackListUpdateError: 
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatBlacklistError'));
				statusbarThreatIcon.setAttribute("class", "status-spywatch-icon");
			break;
			
			case dashboard.kPopupBlockingDisabled:
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatPopupBlocking'));
				statusbarThreatIcon.setAttribute("class", "status-popupblocking-icon");
			break;
			
			case dashboard.kCriticalUpdate:
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatCriticalUpdate'));
				statusbarThreatIcon.setAttribute("class", "status-browser-updates-icon");
			break;
			
			case "extension-update-available" :
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatExtensionUpdate'));
				statusbarThreatIcon.setAttribute("class", "status-browser-update-icon");
			break;
			
			case dashboard.kFullScanNotYetRun :
				// do not trump the blacklist download error threat
				if (threatsQueue.mThreats[dashboard.kBlackListUpdateError] == "red")
					return;			
	
				statusbarThreatIcon.setAttribute("class", "status-spywatch-icon");
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardSpyScanPanelNeverRan'));	
			break;
			
			case dashboard.kActiveXOn:
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatActiveX'));	
				statusbarThreatIcon.setAttribute("class", "sd-sitesettings-icon");
			break;
			
			case dashboard.kIDTheftUpdateDelayTooLong :
				statusbarThreatIcon.setAttribute("class", "identity-guard-icon");
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatIDTheftBlacklistDelay'));	
			break;
			
			case dashboard.kSpywareUpdateDelayTooLong :
				statusbarThreatIcon.setAttribute("class", "status-spywatch-icon");
				statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardStatusThreatSpywareBlacklistDelay'));
			break;
			
			}			
			this.setStatusbarWarningLevel();
	},
	
	
	/*********************************************************************/
	/* SetStatusbarWarningLevel()
	/* Sets the global warning level of the security dashboard
	/* statusbar threat indicator
	/*********************************************************************/
	setStatusbarWarningLevel: function() {
		if (threatsQueue.hasRedThreats()) {
			dashboard.mStatusBarPanel.setAttribute("warningLevel", "2");
		} else if (threatsQueue.hasYellowThreats()) {
			dashboard.mStatusBarPanel.setAttribute("warningLevel", "1");
		} else {
			dashboard.mStatusBarPanel.removeAttribute("warningLevel");
			document.getElementById('currThreatItemIcon').removeAttribute('class');
		}
	},
	
	/*********************************************************************/
	/* resetStatusbarPanelStatus()
	/* Resets the statusbar string to "Netscape Security Center"
	/* and makes it green
	/*********************************************************************/
	resetStatusbarPanelStatus: function() {
		var dashboardPanel = document.getElementById('securityDashboard');
		var statusbarLabel = document.getElementById('securityDashboardLabel');
		if (!statusbarLabel)
			statusbarLabel = DashboardUIController.statusbarLabel;
		
		var statusbarThreatIcon = document.getElementById('currThreatItemIcon');
		if (!statusbarThreatIcon)
			statusbarThreatIcon = DashboardUIController.statusbarThreatIcon;
		
		statusbarLabel.setAttribute('value', this.mDashboardBundle.getString('dashboardTitle'));	
		statusbarThreatIcon.removeAttribute('class');
		dashboardPanel.removeAttribute('warningLevel');
		
	},
	
	debug: function(aString) {
		return;
		dump("\n\nSecurity Dashboard Debug: " + aString + '\n\n');
	},
	
	shouldAutoExpose: function() {
		var shouldExpose = this.mAutoExposeProblem;
		if (shouldExpose != null)
			return shouldExpose;
		else
			return true;		
	},
}

/*******************************************************
 * Prototype for the global threat observer
 * This global observer handles notification for browser
 * threats such as critical update available, spyware
 * full scan not run, blacklist uneffective, and update
 * messages from teh NSIUpdateService
 *******************************************************/
function GlobalThreatObserver() {}

GlobalThreatObserver.prototype = {
	
	 // register the messages we which to be notified of
	 register: function() {
	 	dashboard.debug('registered global observer succesfully');
	 	var observerService = Components.classes["@mozilla.org/observer-service;1"]
                          .getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(this, "critical-update-available", false);
    observerService.addObserver(this, "full-scan-not-run", false);
    observerService.addObserver(this, "idTheft-uneffective", false);
    observerService.addObserver(this, "spyware-uneffective", false);
    observerService.addObserver(this, "Update:Silent:Error", false);
    observerService.addObserver(this, "Update:Silent:Success", false);
    observerService.addObserver(this, "Update:Ended", false); 	
    
    // site security alerts are site specific so they have to be distinguished by window
    // set the attribute on the window so we always have a handle on it
    var activeAppWindow = document.getElementById('main-window');
		if (activeAppWindow) {
			var windowID = this.getCurrentWindowID();
			activeAppWindow.setAttribute('windowID', windowID);
		}
    observerService.addObserver(this, "site-security-alert-" + windowID, false)
    
		// add preference observers
    var pbi = gPrefService.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		pbi.addObserver("app.update.performed", this, false);
		pbi.addObserver("app.update.updatesAvailable", this, false);
		pbi.addObserver("spywatch.lastFullScan", this, false);    
	 },
	 
	unregister: function() {
	 	var observerService = Components.classes["@mozilla.org/observer-service;1"]
                          .getService(Components.interfaces.nsIObserverService);
    observerService.removeObserver(this, "critical-update-available", false);
    observerService.removeObserver(this, "full-scan-not-run", false);
    observerService.removeObserver(this, "idTheft-uneffective", false);
    observerService.removeObserver(this, "spyware-uneffective", false);
    observerService.removeObserver(this, "Update:Silent:Error", false);
    observerService.removeObserver(this, "Update:Silent:Success", false);
    observerService.removeObserver(this, "Update:Ended", false);
    
    var activeAppWindow = document.getElementById('main-window');
		if (activeAppWindow) {
			var windowID = activeAppWindow.getAttribute('windowID');
			observerService.removeObserver(this, "site-security-alert-" + windowID, false) 	
		}
			
		// add preference observers
    var pbi = gPrefService.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		pbi.removeObserver("app.update.performed", this, false);
		pbi.removeObserver("app.update.updatesAvailable", this, false);
		pbi.removeObserver("spywatch.lastFullScan", this, false);    
	},
	
	observe: function(subject, topic, data) {
		// notification for site specific messages
		var activeBrowserWindow = document.getElementById('main-window');
		var windowID = 1;
		if (activeBrowserWindow) {
			windowID = activeBrowserWindow.getAttribute('windowID');
		}
		
		switch(topic) {
			case "full-scan-not-run":
				threatsQueue.add(dashboard.kFullScanNotYetRun, "yellow");
				dashboard.flipStatusbarPanelStatus(dashboard.kFullScanNotYetRun);
			break;
	    
	    case "Update:Silent:Error":
	    	// persist the the silent update errors across all windows
				gPrefService.setBoolPref('silent.update.update_success', false);
				dashboard.mBlackListDownloadError = true;
				threatsQueue.add(dashboard.kBlackListUpdateError, "red");
				// for BLT BUG #: 208544
				// if the root cause of an uneffective blacklist is a silent
				// update, only show that threat
				threatsQueue.remove(dashboard.kIDTheftUpdateDelayTooLong);
				threatsQueue.remove(dashboard.kSpywareUpdateDelayTooLong);
				dashboard.flipStatusbarPanelStatus(dashboard.kBlackListUpdateError);
				if (dashboard.shouldAutoExpose()) {
    			if (dashboard.mDashboardWindow == null)
    				OpenDashboardWindow('fade');
    			else if (dashboard.mDashboardWindowLoaded)
	     				dashboard.mDashboardWindow.UpdateSecurityDashboard();
	    	}
	    break;
	    	
	    case "Update:Silent:Success":
	     	gPrefService.setBoolPref('silent.update.update_success', true);
	     	dashboard.mBlackListDownloadError = false;
	     	threatsQueue.remove(dashboard.kIDTheftUpdateDelayTooLong);
	     	threatsQueue.remove(dashboard.kSpywareUpdateDelayTooLong);
	     	threatsQueue.remove(dashboard.kBlackListUpdateError);
	     	if (dashboard.mDashboardWindow != null) {
	     			if (dashboard.mDashboardWindowLoaded)
	     				dashboard.mDashboardWindow.UpdateSecurityDashboard();
	     		} 
	    	dashboard.flipToNextThreat();

	    break;
	    	
	    case "Update:Ended" :
	    		//dump(data);
	    break;
	    
	   	case "critical-update-available":	
				threatsQueue.add(dashboard.kCriticalUpdate, "red");
				dashboard.flipStatusbarPanelStatus(dashboard.kCriticalUpdate);
			break;
			
			case "extension-update-available":
				threatsQueue.remove(dashboard.kCriticalUpdate);
			break;
			
			case "idTheft-uneffective":
				dashboard.flipStatusbarPanelStatus(dashboard.kIDTheftUpdateDelayTooLong);
				threatsQueue.add(dashboard.kIDTheftUpdateDelayTooLong, "yellow");
			break;
			
			case "spyware-uneffective":
				dashboard.flipStatusbarPanelStatus(dashboard.kSpywareUpdateDelayTooLong);
				threatsQueue.add(dashboard.kSpywareUpdateDelayTooLong, "yellow");
			break;
			
			case "nsPref:changed" :
				// reset the critical warning if the update was performed
				if (data == "app.update.performed" && gPrefService.getBoolPref('app.update.performed')) {
					// an update was performed so remove it as a threat in the queue		
						threatsQueue.remove(dashboard.kCriticalUpdate);
						if (dashboard.mDashboardWindow != null)
							if (dashboard.mDashboardWindowLoaded)
								dashboard.mDashboardWindow.UpdateSecurityDashboard();					
					}				
				// set up the UI if an update is available	
				if (data == "app.update.updatesAvailable") {
					if (gPrefService.getBoolPref('app.update.updatesAvailable')) {
						if (dashboard.CheckBrowserUpdatePrefs()) {
							if (dashboard.mDashboardWindow != null)
								if (dashboard.mDashboardWindowLoaded)
									dashboard.mDashboardWindow.UpdateSecurityDashboard();
						}
					}
				}
				
				if (data == "spywatch.lastFullScan") {	
					if (gPrefService.getIntPref(data) > 0) {
						threatsQueue.remove(dashboard.kFullScanNotRun);
						dashboard.init();
						dashboard.flipToNextThreat();
						if (dashboard.mDashboardWindow != null) {
							if (dashboard.mDashboardWindowLoaded)
								dashboard.mDashboardWindow.UpdateSecurityDashboard();
						}
					}
				}	
			break;
						
			case "site-security-alert-" + windowID:	
    		// there is no security alert
    		if (data == "-1") {
	    		// if there are no site specific alerts remove any of these alerts from the queue
	    		threatsQueue.remove(dashboard.kSpyware);
	    		threatsQueue.remove(dashboard.kPhishing);
	    		threatsQueue.remove(dashboard.kUnverifiedSite);
	    	}
    		
    		// spyware warning    		
    		if (data == "spyware") {	
	    		threatsQueue.add(dashboard.kSpyware, "red");
	    		threatsQueue.remove(dashboard.kUnverifiedSite);
	    		threatsQueue.remove(dashboard.kPhishing); 		
	    		dashboard.flipStatusbarPanelStatus(dashboard.kSpyware);	
	    	}
	    	
	    	if (data == "phishing") {
    			threatsQueue.add(dashboard.kPhishing, "red");
    			threatsQueue.remove(dashboard.kUnverifiedSite);
    			threatsQueue.remove(dashboard.kSpyware);
    			dashboard.flipStatusbarPanelStatus(dashboard.kPhishing);
	    	}
	    	
	    	if (data == "unverified-site") {
					threatsQueue.add(dashboard.kUnverifiedSite, "yellow");
					threatsQueue.remove(dashboard.kSpyware);
					threatsQueue.remove(dashboard.kPhishing);
					dashboard.flipStatusbarPanelStatus(dashboard.kUnverifiedSite);
				}
				
				if (data == "popup-blocking-disabled") {
	    		threatsQueue.add(dashboard.kPopupBlockingDisabled, "yellow");
	    		dashboard.flipStatusbarPanelStatus(dashboard.kPopupBlockingDisabled);
				}			
			break;
	   	    
	    default:
	    	break;
			
		}
	},
	
	getCurrentWindowID: function() {
		var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
		var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
		var enumerator = windowManagerInterface.getEnumerator(null);
		var numWindows = 1;
		while (enumerator.hasMoreElements()) {
			enumerator.getNext();
			numWindows++;
		}
				
		return numWindows;	
	},
}
