var helper =  {
  // CID of objects
  CID_RDFSVC : '@mozilla.org/rdf/rdf-service;1',

  // RDF tags
  RDF_PREFIX : 'root:Pest_',
  RDF_NAME : 'root:Name',
  RDF_AUTHOR : 'root:Author',
  RDF_DESC : 'root:Description',
  RDF_CATEGORY : 'root:Category',
  RDF_DATE : 'root:ReleaseDate',
  
  m_tree : 0,         // reference to the item tree
  m_bShowAll : true,  // show all items or show undeleted items
  m_pest : 0,         // current pest to be displayed
  m_collection : 0,   // collection of items for the selected pest
  m_strbundle : 0,    // string bundle for dynamically set strings
  
  load : function () {
    this.m_strbundle = document.getElementById('sw_strings');
  
    if ("arguments" in window) {
      var params = window.arguments[0];
      if (params instanceof Components.interfaces.nsIDialogParamBlock) {
        var id = params.GetString(0);
        var objArray = params.objects;
        
        objArray.QueryInterface(Components.interfaces.nsIArray);
        this.m_collection = objArray.queryElementAt(0, Components.interfaces.nsISpywareListCollection);

        if (0 != this.m_collection) {
          var rdfService = Components.classes[this.CID_RDFSVC]
                            .getService(Components.interfaces.nsIRDFService);
                            
          var rdf = this.m_collection.getDataSource();
          this.m_pest = rdfService.GetResource(this.RDF_PREFIX + id);
          
          // Pest Name
          var litValue = this.m_collection.name(this.m_pest);
          var elem = document.getElementById('spywareScanHelpPest');
          (elem.QueryInterface(Components.interfaces.nsIDOMXULDescriptionElement)).value=litValue;

          // Category
          litValue = this.m_collection.category(this.m_pest);
          elem = document.getElementById('spywareScanHelpCategory');
          (elem.QueryInterface(Components.interfaces.nsIDOMXULDescriptionElement)).value=litValue;
          
          // Description
          litValue = this.m_collection.description(this.m_pest);
          elem = document.getElementById('spywareScanHelpDescription');
          (elem.QueryInterface(Components.interfaces.nsIDOMXULDescriptionElement)).value=litValue;
          
          // Author
          litValue = this.m_collection.author(this.m_pest);
          elem = document.getElementById('spywareScanHelpAuthor');
          (elem.QueryInterface(Components.interfaces.nsIDOMXULDescriptionElement)).value=litValue;
          
          // Release Date
          litValue = this.m_collection.releaseDate(this.m_pest);
          elem = document.getElementById('spywareScanHelpReleaseDate');
          (elem.QueryInterface(Components.interfaces.nsIDOMXULDescriptionElement)).value=litValue;
          
          // Threat
          litValue = this.m_collection.threat(this.m_pest);
          elem = document.getElementById('spywareScanHelpThreat');
          var textElement = document.createTextNode(litValue);
          elem.appendChild(textElement);
          
          // Risk
          litValue = this.m_collection.risk(this.m_pest);
          elem = document.getElementById('spywareScanHelpRisk');
          textElement = document.createTextNode(litValue);
          elem.appendChild(textElement);
          
          // Advice
          litValue = this.m_collection.advice(this.m_pest);
          elem = document.getElementById('spywareScanHelpAdvice');
          textElement = document.createTextNode(litValue);
          elem.appendChild(textElement);
          
          // get a reference to the tree element
          m_tree = document.getElementById('itemTree');
          
          this.loadTree(m_tree);
        }      
      }
    }
  },
  
  itemview : function (bAll) {
    this.m_bShowAll = bAll;
    
    this.clearTree();
    this.loadTree(this.m_tree);
  },
  
  clearTree : function () {
    var element = document.getElementById('treeitem.process');
    this.clearTreeItem(element);
    
    element = document.getElementById('treeitem.registry');
    this.clearTreeItem(element);

    element = document.getElementById('treeitem.files');
    this.clearTreeItem(element);
  },
  
  clearTreeItem : function (element) {
    var id = element.getAttribute('id') + '.children';
    var subElement = document.getElementById(id);
    
    if (subElement) {
      var child;
      while (subElement.hasChildNodes()) {
        child = subElement.lastChild;
        subElement.removeChild(child);
      }
      
      if (element.hasAttribute('container')) {
        element.removeAttribute('container')
        element.removeChild(subElement);
      }
    }
  },
  
  loadTree : function (tree) {
    // Get the individual items that make up the pest
    var items = this.m_collection.items(this.m_pest);
    if (items instanceof Components.interfaces.nsISimpleEnumerator) {
      var rdfItem;
      var attributes, type, name, bDeleted, msg;
      var treeElement;
      while (items.hasMoreElements()) {
        rdfItem = items.getNext();
        if (rdfItem instanceof Components.interfaces.nsIRDFResource) {
          type  = this.m_collection.itemType(rdfItem);
          attributes = this.m_collection.itemAttributes(rdfItem);
          switch(type) {
            case 0: // unknown
            case 1: // process
              treeElement = document.getElementById('treeitem.process');
            case 2: // registry
              treeElement = document.getElementById('treeitem.registry');
            case 3: // file
              treeElement = document.getElementById('treeitem.files');
          }

          if (type > 0) {
            name = this.m_collection.itemName(rdfItem);
            bDeleted = this.m_collection.itemDeleted(rdfItem);
           
            if (this.m_bShowAll || !bDeleted) {
              if (!bDeleted) {
                msg = this.m_collection.itemErrorMsg(rdfItem);
              }
              this.addTreeChild(tree, treeElement, name, bDeleted, msg, attributes);
            }
          }
        }
      }
    }
  },

  addTreeChild : function (tree, elem, name, bDeleted, msg, attributes) {
    // First determine if the current element is already a container
    var childElement;
    if (!elem.hasAttribute('container')) {
      // if not, add the container attribute
      elem.setAttribute('container', true);
      childElement = document.createElement('treechildren');
      childElement.setAttribute('id', elem.getAttribute('id') + '.children');
    } else {
      childElement = document.getElementById(elem.getAttribute('id') + '.children');
    }
    
    // Create the treeChildren element and its children, and add to the container element
    var childItem = document.createElement('treeitem');
    childElement.appendChild(childItem);
    
    var childRow = document.createElement('treerow');
    childItem.appendChild(childRow);
    
    var childCell = document.createElement('treecell');
    childCell.setAttribute('label', name);
    childRow.appendChild(childCell);
        
    var childCell = document.createElement('treecell');
    if (bDeleted) {
      childCell.setAttribute('label', this.m_strbundle.getString('helpdeleted'));
    } else {
      childCell.setAttribute('label', this.m_strbundle.getString('helpnotdeleted') + ': ' + msg);
    }
    childRow.appendChild(childCell);
    
    // Return the treeChildren element
    elem.appendChild(childElement);
    //tree.treeBoxObject.invalidate();
    return childElement;
  }
}