var spyWareScanner = {
	DEBUG: true,
  
  // Preference tags
  kprefSpywareAutoDelete        : "spywatch.autoDeleteSpyware",
  kprefSpywareBlacklistUpdate   : "spywatch.blacklistUpdateFreq",
	kprefSpywareInitialScan       : "spywatch.initialScan",
	kprefSpywareIsInstalled       : "spywatch.isInstalled",
  kprefSpywareIsScanningFile    : "spywatch.scanning.file",
  kprefSpywareIsScanningSystem  : "spywatch.scanning.system",
  kprefSpywareLastScan          : "spywatch.lastFullScan",
  kprefSpywareNextUpdate        : "spywatch.nextUpdate",
  kprefSpywareNextUpdateDiff    : "spywatch.nextUpdateDiff",
  kprefSpywareNotifyDeleted     : "spywatch.notifySpywareDeleted",
  kprefSpywareScanDownloads     : "spywatch.scanDownloads",
  kprefSpywareShowWarning       : "spywatch.showWarning",
  kprefSpywareStatus            : "spywatch.install",
  kSpywareStatusUpdate          : 4,

	debug : function(inMsg) {
		if (this.DEBUG)
			dump('spywareutility.js : ' + inMsg + '\n');
	},
	
	init : function() {

	  // BLT Bug #197351
	  // Run this check first to ensure the PestPatrol component is available
	  var bIsAvailable = false;
 		const cid = "@netscape.com/browser/antispyware-service;1";
		var options = Components.classes[cid].createInstance(Components.interfaces.nsISpywareOptions);
		if (options) {
		  bIsAvailable = options.isAvailable;
		}
	
    var bInstalled = false;
    if (bIsAvailable) {
	    // ensure the appropriate pref settings are initialized
 	    gPrefService = Components.classes["@mozilla.org/preferences-service;1"]
                                  .getService(Components.interfaces.nsIPrefBranch);

      // ensure all prefs are set to some value.  If not, define a default.
      // AAM: default determined by whether the spywatch component is installed.
      if ( !gPrefService.getPrefType(this.kprefSpywareIsInstalled) ) {
        gPrefService.setBoolPref(this.kprefSpywareIsInstalled, bInstalled);
      } else {
        bInstalled = gPrefService.getBoolPref(this.kprefSpywareIsInstalled);
      }
    }

    // spyware site config
    if ( !gPrefService.getPrefType(this.kprefSpywareShowWarning) ) {
      gPrefService.setBoolPref(this.kprefSpywareShowWarning, true);
    }
    
    if ( !gPrefService.getPrefType(this.kprefSpywareScanDownloads) ) {
      gPrefService.setBoolPref(this.kprefSpywareScanDownloads, bInstalled);
    }

    if ( !gPrefService.getPrefType(this.kprefSpywareAutoDelete) ) {
      gPrefService.setBoolPref(this.kprefSpywareAutoDelete, bInstalled);
    }

    if ( !gPrefService.getPrefType(this.kprefSpywareNotifyDeleted) ) {
      gPrefService.setBoolPref(this.kprefSpywareNotifyDeleted, bInstalled);
    }

    if ( !gPrefService.getPrefType(this.kprefSpywareBlacklistUpdate) ) {
      gPrefService.setIntPref(this.kprefSpywareBlacklistUpdate, 86400000);
    }

    if ( !gPrefService.getPrefType(this.kprefSpywareLastScan) ) {
      gPrefService.setIntPref(this.kprefSpywareLastScan, 0);
    }

    if ( !gPrefService.getPrefType(this.kprefSpywareIsScanningFile) ) {
      gPrefService.setBoolPref(this.kprefSpywareIsScanningFile, false);
    }

    if ( !gPrefService.getPrefType(this.kprefSpywareIsScanningSystem) ) {
      gPrefService.setBoolPref(this.kprefSpywareIsScanningSystem, false);
    }
    
    return bIsAvailable;  // return availability
	},

  performFullScan : function() {
    // check is we are in the middle of an update
 		const cid = "@netscape.com/browser/antispyware-service;1";
		obj = Components.classes[cid].createInstance(Components.interfaces.nsISpywareUpdate);
		if (obj.isUpdating) {
      window.openDialog('chrome://browser/content/spywarescan/updatenotify.xul',
                        undefined,'modal,centerscreen');
      return;
		}

   	const ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
            .getService(Components.interfaces.nsIWindowWatcher);

    //Merc - TO
    //BLT 213312
    //alwaysRaised causes issues with focus.
    ww.openWindow(null, 'chrome://browser/content/spywarescan/systemscan.xul',
                      "_blank", "chrome,centerscreen,dialog=no", null);
  },
  
  firstFullScan : function() {
    // on browser startup, check if a full scan has been performed and initiate
    if (!this.isInstalled())
    {
      // return immediately if not installed
      return false;
    }
    
    var bInitialized = false;
    if ( gPrefService.getPrefType(this.kprefSpywareInitialScan) ) {
      if ( gPrefService.getBoolPref(this.kprefSpywareInitialScan) ) {
        bInitialized = true;
      }
    } else {
      // ensure it is set appropriately in the future
      gPrefService.setBoolPref(this.kprefSpywareInitialScan, false);
    }
    
    var retValue = false;
    if (!bInitialized) {
      this.performFullScan();
      retValue = true;

      // After the scan, check if the last full scan time is set to verify
      // that a full scan has completed.
      // AAM: Note - if kprefSpywareLastScan does not exist, then the scan did
      //      not complete.
      if ( gPrefService.getPrefType(this.kprefSpywareLastScan) ) {
        if ( gPrefService.getIntPref(this.kprefSpywareLastScan) ) {
          gPrefService.setBoolPref(this.kprefSpywareInitialScan, true);
        }
      }
    }
    // This should only run on the very first invocation of the browser
    // Therefore at this point force the flag on
    gPrefService.setBoolPref(this.kprefSpywareInitialScan, true);
    return retValue;
  },

	currentStatus : function() {
		var retValue = 0;
		
	 	var gPrefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

	  if (gPrefService.getPrefType("spywatch.install") == gPrefService.PREF_INT) {
			retValue = gPrefService.getIntPref("spywatch.install");				  	
	  }
	  
	  return retValue;
	},
		
	isInstalled : function() {
		if (!gPrefService.getPrefType(this.kprefSpywareIsInstalled))
		{
		  //gPrefService.setBoolPref(this.kprefSpywareIsInstalled, false);
		  return false;
		} else {
		  return gPrefService.getBoolPref(this.kprefSpywareIsInstalled);
		}
	},
	
	update : function() {
		var retValue = false;
		
		if (!this.isInstalled()) {
		  // not installed - abort
		  return retValue;
		}

    // ensure the time difference is set
    // if not defined, defaults to 1 week
    if ( !gPrefService.getPrefType(this.kprefSpywareNextUpdateDiff) ) {
      gPrefService.setIntPref(this.kprefSpywareNextUpdateDiff, (60*60*24*7));
    }

    // Get instance of updater
		const cid = "@netscape.com/browser/antispyware-service;1";
		obj = Components.classes[cid].createInstance(Components.interfaces.nsISpywareUpdate);
    
    // get the timestamp in seconds of the last update
    var nextUpdate = obj.nextUpdate;
    
    var now = new Date();
    if ( (0 == nextUpdate) || (nextUpdate - (now.valueOf()/1000) < 0 ) )
    {
		  try {

	      obj.update();
			  retValue = true;

		  } catch (err) {
		  }
    } 

		return retValue;
	}
}
