
var systemscan = {
  // list of possible states that the window can be in
  stateStart : 0,
  stateScanningMemory : 10,
  stateScanningRegistry : 11,
  stateScanningFiles : 12,
  stateScanComplete : 20,
  stateScanFound : 30,
  stateRemoving : 40,
  stateProblems : 50,
  stateSummary : 60,
  stateRebootError : 70,

  // CID of objects
  CID_SCANNER : '@netscape.com/browser/antispyware-service;1',
  CID_COLLECTION : '@netscape.com/browser/spyware-collection;1',
  CID_ITEM : '@netscape.com/browser/spyware-item;1',
  CID_PEST : '@netscape.com/browser/spyware-pest;1',
  CID_CLEANER : '@netscape.com/browser/spyware-cleaner;1',
  CID_LOCALDRV : '@netscape.com/browser/localdisk;1',
  CID_PREFSVC : '@mozilla.org/preferences-service;1',
  CID_PARAMITEM : '@mozilla.org/embedcomp/dialogparam;1',
  CID_ARRAY : '@mozilla.org/array;1',
  CID_RDFSVC : '@mozilla.org/rdf/rdf-service;1',
  CID_RDFCONTAINER : '@mozilla.org/rdf/container;1',
  CID_RDFCONTUTIL : '@mozilla.org/rdf/container-utils;1',

  // RDF tags
  RDF_PESTS : 'root:Pests',
  RDF_DELETE : 'root:bDelete',
  RDF_NAME : 'root:Name',
  RDF_CLEANED : 'root:bCleaned',
  RDF_ERRMSG : 'root:ErrorMsg',
  RDF_ID : 'root:ID',

  // Max rows in a list box
  kMaxRows : 10,

  // current state of the window
  currentState : this.stateStart,

  // track the drives that will be scanned
  drives : new Array(),
  nTotalDrives : 0,
  nSelectedDrives : 0,
  bAllDrives : true,

  // track files scanned and problems found
  nTotalFiles : 0,
  nTotalHits : 0,

  // track spyware items to be cleaned
  bAllSpyware : true,
  bNoSpyware : false,
  nSpywareToClean : 0,

  // Track which items to clean
  nIgnoredCount : 0,

  // Results of cleaning
  bAllCleaned : true,
  bNeedReboot : false,
  nTotalCleaned : 0,
  bFullScanComplete : false,

  // stringbundle - spywarescan.properties
  strbundle : null,

  // COM objects
  scanner : null,
  collection : null,
  cleaner : null,
  gPrefService : null,

  // Observers
  observer : null,

  // Async timer flags for UI
  bAsyncUpdate : false,
  bAsyncShutdown : false,
  bAsyncID : 0,

  initDrives : function () {
    // set up the initial drives to be scanned
    var diskInfo;
    try {
      diskInfo = Components.classes[this.CID_LOCALDRV].createInstance(Components.interfaces.nsILocalDisk);
    } catch (err) {
      bAllDrives = false;
      return;
    }

    if (diskInfo) {
      var drive;
      for (var index = new String('A');
           index <= 'Z';
           index = String.fromCharCode(index.charCodeAt(0) + 1)) {
        drive = index + ':\\';
        if (diskInfo.isDiskFixed(drive)) {
          this.drives.push(drive);
          this.nTotalDrives = this.nTotalDrives + 1;
        }
      }
      this.nSelectedDrives = this.nTotalDrives;
      this.bAllDrives = true;
    }
  },

  cleanup : function() {
  	if (! this.ISDUPLICATE) {
	  	this.gPrefService.setBoolPref('spywatch.window.open',false);
	    this.gPrefService.setBoolPref('spywatch.scanning.system', false);
	  }
  },

  init : function() {
    this.strbundle = document.getElementById('sw_strings');

   	this.gPrefService = Components.classes[this.CID_PREFSVC]
                                .getService(Components.interfaces.nsIPrefBranch);

		// check if pref set indicating current window exists
		if ( this.gPrefService.getPrefType('spywatch.window.open') && this.gPrefService.getBoolPref('spywatch.window.open') ) {
			this.ISDUPLICATE = true;
			window.close();

		} else {
			this.gPrefService.setBoolPref('spywatch.window.open',true);
		}

		// if set, find window and bring forword, destroy this one
		// otherwise, set preference
    try {
      if (!this.scanner) {
        this.scanner = Components.classes[this.CID_SCANNER].createInstance(Components.interfaces.nsISpywareSystemScan);
      }
    } catch (e) {
      //this.logError(e, 'unable to create nsSpywareScanner');
      throw e;
    }

    try {
      if (!this.collection) {
        this.collection = Components.classes[this.CID_COLLECTION].createInstance();
      }
    } catch (e) {
      //this.logError(e, 'unable to create nsSpywareCollection');
      throw e;
    }

    var syscollection = this.collection.QueryInterface(Components.interfaces.nsISpywareScanCollection);

    nIgnoredCount = 0;  // start with no items ignored

    this.scanner.initializeSystemScan(syscollection);

    this.initDrives();

    this.currentState = this.stateStart;
    this.updateDisplay();
  },

	show_rdf_ds : function(ds) {
	  var rdfService = Components.classes[this.CID_RDFSVC].
                       getService(Components.interfaces.nsIRDFService);
    var t="";
    var all_res=ds.GetAllResources();
    for(var r=0; all_res.hasMoreElements(); r++)
    {
      //if(t.length>5000) break;  // comment out for unlimited output
      var res=all_res.getNext();
      // res is a resource
      if (res instanceof Components.interfaces.nsIRDFResource)
      {
        var tr=r + " R: "+res.Value+"\n";
        var ps= ds.ArcLabelsOut(res);
        while (ps.hasMoreElements())
        {
          var predicate = ps.getNext();
          // predicate is a resource
          if (predicate instanceof Components.interfaces.nsIRDFResource)
          {
            tr+="  -R:  "+predicate.Value+"\n";
            var ts= ds.GetTargets(res, predicate, true);
            while (ts.hasMoreElements())
            {
              var target = ts.getNext();
              // target is either a resource, a literal or a typed-literal
              if (target instanceof Components.interfaces.nsIRDFResource)
                tr+="   >R: "+target.Value+"\n";
              else
              if (target instanceof Components.interfaces.nsIRDFLiteral)
                tr+="    >L: "+target.Value+"\n";
              else
              if (target instanceof Components.interfaces.nsIRDFBlob)
                tr+="    >BLOB length:"+target.length+"\n";
              else
              if (target instanceof Components.interfaces.nsIRDFInt)
                tr+="    >INT:"+target.Value+"\n";
              else
              if (target instanceof Components.interfaces.nsIRDFDate)
                tr+="    >DATE: "+target.Value+"\n";
              else
              if (target instanceof Components.interfaces.nsIRDFNode)
                tr+="    >Node \n";
              else
                tr+="    >Unknown\n";
            }
          } // end predicate-res
       } // end ps
       t+=tr+"\n";
      } else // should never happen
      if (res instanceof Components.interfaces.nsIRDFLiteral)
      {
        t+=r + "L: "+res.Value+"\n";
      } else
        t+=r + "?: "+"\n";
    }
    return t;
  },

  showDeckLayer : function(deckId, layerId) {
    var deck = document.getElementById(deckId);
    for (var i = 0; i < deck.childNodes.length; i++) {
      if (deck.childNodes[i].id == layerId) {
        deck.selectedIndex = i;
        return;
      }
    }
  },

  updateRemoveButtonsDisplay : function() {
    this.nIgnoredCount = this.nTotalHits - this.getRemoveCount();
  
    // First set applicable flags
    if ( (0 == this.nTotalHits) || (0 == (this.nTotalHits - this.nIgnoredCount)) ) {
      this.bAllSpyware = false;
      this.bNoSpyware = true;
    }
    else if ( (0 == this.nIgnoredCount) && (this.nTotalHits > 0) ) {
      // all items have been deselected
      this.bAllSpyware = true;
      this.bNoSpyware = false
    }
    else {
      this.bAllSpyware = false;
      this.bNoSpyware = false;
    }

    // set enabled/disable state of the check/uncheck all buttons
    var btnCheckAll = document.getElementById('scanRemoveSpywareButtons.checkall')
    var btnUncheckAll = document.getElementById('scanRemoveSpywareButtons.uncheckall');
    if ( (btnCheckAll instanceof Components.interfaces.nsIDOMXULCheckboxElement) &&
         (btnUncheckAll instanceof Components.interfaces.nsIDOMXULCheckboxElement) ) {
    }

    if (this.bAllSpyware) {
      this.showDeckLayer('removeButtonDeck', 'scanRemoveAllButton');
      btnCheckAll.disabled = true;
      btnUncheckAll.disabled = false;
      document.getElementById('scanRemoveAllButton').focus();

    } else if (this.bNoSpyware) {
      this.showDeckLayer('removeButtonDeck', 'scanRemoveDisabledButton');
      btnCheckAll.disabled = (0 == this.nTotalHits);
      btnUncheckAll.disabled = true;
      document.getElementById('scanRemoveCloseButton').focus(); // focus on Close button

    } else {
      this.showDeckLayer('removeButtonDeck', 'scanRemoveSelectedButton');
      btnCheckAll.disabled = false;
      btnUncheckAll.disabled = false;
      document.getElementById('scanRemoveSelectedButton').focus();
    }
  },

  updateDisplay : function() {
    if (STATE_MINIMIZED == window.windowState) {
      // If we are minimized, don't change anything in the UI
      // Otherwise the window is restored
      return;
    }
    
    switch (this.currentState) {
      case this.stateStart:
        this.showDeckLayer('descriptionDeck', 'startscanDescription');
        this.showDeckLayer('buttonDeck', 'startscanButtons');

        var btnStart = document.getElementById('startsystemscan');
        // set focus to start button
        btnStart.focus();
        
        // disable the scan button if no drives selected
        btnStart.disabled = (0 == this.nSelectedDrives);

        if (this.bAllDrives) {
          this.showDeckLayer('actionDeck','scanreadyallAction');
        } else {
          var showCount = this.strbundle.getString('selectedscanned');
          showCount = showCount.replace('%1', this.nSelectedDrives);
          showCount = showCount.replace('%2', this.nTotalDrives);
          document.getElementById('spywatch.action.count.drivesscanned').value = showCount;

          this.showDeckLayer('actionDeck','scanreadyselectedAction');
        }
        break;

      case this.stateScanningMemory:
        this.showDeckLayer('descriptionDeck', 'scanningDescription');
        this.showDeckLayer('actionDeck', 'scanmemoryAction');
        this.showDeckLayer('buttonDeck', 'scanningButtons');
        
        // set focus on stop button
        //var btnStop = document.getElementById('stopsystemscan').focus();
        break;

      case this.stateScanningRegistry:
        this.showDeckLayer('descriptionDeck', 'scanningDescription');
        this.showDeckLayer('actionDeck', 'scanregistryAction');
        this.showDeckLayer('buttonDeck', 'scanningButtons');

        // set focus on stop button
        //document.getElementById('stopsystemscan').focus();
        break;

      case this.stateScanningFiles:
        this.showDeckLayer('descriptionDeck', 'scanningDescription');
        this.showDeckLayer('buttonDeck', 'scanningButtons');

        // set focus on stop button
        //document.getElementById('stopsystemscan').focus();

        var desc=document.getElementById('spywatch.action.count.filesscanned');
        this.getScanResults(desc);

        this.showDeckLayer('actionDeck', 'scanfilesAction');
        break;

      case this.stateScanComplete:
        this.showDeckLayer('descriptionDeck','scancompleteDescription');
        this.showDeckLayer('buttonDeck','scanCompleteButtons');
        
        // set focus on OK button
        document.getElementById('completesystemscan').focus();

        var desc = document.getElementById('spywatch.action.count.totalfiles');
        this.getScanResults(desc);

        this.showDeckLayer('actionDeck','scanresultsokAction');
        break;

      case this.stateScanFound:
        this.showDeckLayer('descriptionDeck', 'scanresultsDescription');

        var desc = document.getElementById('spywatch.action.list.totalfiles');
        this.getScanResults(desc);

        var table = document.getElementById('scanlistfound.listbox');
        var listCollection = this.collection.QueryInterface(Components.interfaces.nsISpywareListCollection);
        var ds = listCollection.getDataSource();

        table.database.AddDataSource(ds);
        table.builder.rebuild();

        this.showDeckLayer('actionDeck', 'scanresultslistAction');

        this.updateRemoveButtonsDisplay()
        this.showDeckLayer('buttonDeck', 'scanRemoveSpywareButtons');
        
        break;

      case this.stateRemoving:
        if (this.bAllSpyware) {
          this.showDeckLayer('descriptionDeck', 'scanremoveallDescription');
          this.showDeckLayer('actionDeck', 'scanremoveallAction');
        } else {
          this.showDeckLayer('descriptionDeck', 'scanremoveselectedDescription');
          this.showDeckLayer('actionDeck', 'scanremoveselectedAction');
        }

        this.showDeckLayer('buttonDeck', 'scanRemoveCancelButtons');
        //document.getElementById('scanremovecancelbutton').focus();
        break;

      case this.stateProblems:
        this.showDeckLayer('descriptionDeck', 'scanresultslistDescription');
        
        var table = document.getElementById('scansummary.listbox');
        var listCollection = this.collection.QueryInterface(Components.interfaces.nsISpywareListCollection);
        var ds = listCollection.getDataSource();

        if (table.getRowCount() == 0)
        {
          this.fillSummaryList(ds,table);
        }
          
        this.showDeckLayer('actionDeck', 'scansummarylistAction');
        this.showDeckLayer('buttonDeck', 'scanRemoveContinueButtons');
        //document.getElementById('scanremovecontinuebutton').focus();
        break;

      case this.stateSummary:
        var desc;
        
        this.showDeckLayer('descriptionDeck', 'scansummaryDescription');
        if (this.bAllCleaned) {
          if (this.bAllSpyware) {
            this.showDeckLayer('scansummaryDescription.caption', 'scansummaryDescription.all');
            this.showDeckLayer('actionDeck', 'scancompleteAction.allremoved');
            desc = document.getElementById('spywatch.action.all.count.problems');
            
          } else {
            this.showDeckLayer('scansummaryDescription.caption', 'scansummaryDescription.selected');
            this.showDeckLayer('actionDeck', 'scancompleteAction.selectedremoved');
            desc = document.getElementById('spywatch.action.selected.count.problems');

          }
        } else {
          if (this.bAllSpyware) {
            this.showDeckLayer('scansummaryDescription.caption', 'scansummaryDescription.all.someremoved');
            this.showDeckLayer('actionDeck', 'scancompleteAction.allpartial');
            desc = document.getElementById('spywatch.action.allpartial.count.problems');

          } else {
            this.showDeckLayer('scansummaryDescription.caption', 'scansummaryDescription.selected.someremoved');
            this.showDeckLayer('actionDeck', 'scancompleteAction.selectedpartial');
            desc = document.getElementById('spywatch.action.selectedpartial.count.problems');

          }
        }

        var template;
        if (this.bNeedReboot) {
          this.showDeckLayer('scansummaryDescription.info', 'scansummaryDescription.restartclose');

          if (this.bAllCleaned) {
            if (1==this.nTotalCleaned) {
              template = this.strbundle.getString('summarycountonerestart');
            } else {
              template = this.strbundle.getString('summarycountallrestart');
            }
          } else {
            template = this.strbundle.getString('summarycountsomerestart');
          }

        } else {
          this.showDeckLayer('scansummaryDescription.info', 'scansummaryDescription.close');

          if (this.bAllCleaned) {
            if (1==this.nTotalCleaned) {
              template = this.strbundle.getString('summarycountone');
            } else {
              template = this.strbundle.getString('summarycountall');
            }
          } else {
            template = this.strbundle.getString('summarycountsome');
          }
        }
        template = template.replace('%1', this.nTotalCleaned)
        template = template.replace('%2', this.nSpywareToClean);
        desc.value = template;

        if (this.bNeedReboot) {
          this.showDeckLayer('buttonDeck', 'scanRemoveRestartButtons');
          document.getElementById('scanremoverestartbutton').focus();
        } else {
          this.showDeckLayer('buttonDeck', 'scanCompleteButtons');
          document.getElementById('completesystemscan').focus();
        }
        break;
    }
    this.setListboxRows();
    window.sizeToContent();
  },

  getScanResults : function(descElement) {
    // First get the updated counts
    var stats = this.collection.QueryInterface(Components.interfaces.nsISpywareScanStatistics);
    this.nTotalHits = stats.totalSpywareFiles;
    this.nTotalFiles = stats.totalFilesScanned;

    var template, template2, textClass;
    
    // process files scanned
    if (1 == this.nTotalFiles) {
      template = this.strbundle.getString('onefilescanned');
    } else {
      template = this.strbundle.getString('filesscanned');
      template = template.replace('%1', this.nTotalFiles);
    }
    
    // process spyware detected
    switch (this.nTotalHits) {
      case 0:
        template2 = this.strbundle.getString('scannednonefound');
        break;
      
      case 1:
        template2 = this.strbundle.getString('scannedonefound');
        break;
      
      default:
        template2 = this.strbundle.getString('scannedsomefound');
        template2 = template2.replace('%2', this.nTotalHits);
        break;
    }
    
    // determine the class type for the spanned template text
    if (0 == this.nTotalHits) {
      textClass = 'spywareStatusTextOK';
    } else {
      textClass = 'spywareStatusTextError';
    }

    var childNodes = descElement.childNodes;
    var i, desc;
    desc = childNodes.item(0);
    if (desc instanceof Components.interfaces.nsIDOMXULDescriptionElement) {
      desc.value = template;
    }
    
    desc = childNodes.item(1);
    if (desc instanceof Components.interfaces.nsIDOMElement) {
      desc.setAttribute('class', textClass);
    }
    if (desc instanceof Components.interfaces.nsIDOMXULDescriptionElement) {
      desc.value = template2;
    }    
  },

  setListboxRows : function() {
    switch(this.currentState) {
      case this.stateScanFound:
        var listbox = document.getElementById('scanlistfound.listbox');
        if (listbox) {
          listbox.setAttribute('rows', this.kMaxRows);
        }
        break;
        
      case this.stateProblems:
        var listbox = document.getElementById('scansummary.listbox');
        if (listbox) {
          listbox.setAttribute('rows', this.kMaxRows);
        }
        break;

      default:
        var listbox = document.getElementById('scanlistfound.listbox');
        if (listbox) {
          listbox.setAttribute('rows', 0);
        }
        listbox = document.getElementById('scansummary.listbox');
        if (listbox) {
          listbox.setAttribute('rows', 0);
        }
        break;
    }
  },

  updateDisplayAsync : function() {
    this.bAsyncID = 0;
    var stats = this.collection.QueryInterface(Components.interfaces.nsISpywareScanStatistics);
    this.nTotalHits = stats.totalSpywareFiles;
    this.nTotalFiles = stats.totalFilesScanned;

    if (stats.isClosed && !this.bAsyncShutdown) {
      switch (this.currentState) {
        case this.stateScanningMemory:
          this.scanner.scanMemory = false;
          this.currentState = this.stateScanningRegistry;
          this.updateDisplay();
          this.scanner.scanRegistry = true;
          stats.reset();
          this.scanner.execute();
          break;

        case this.stateScanningRegistry:
          this.scanner.scanRegistry = false;
          this.currentState = this.stateScanningFiles;
          this.updateDisplay();
          this.scanner.scanDrives = true;

          for (var i=0; i<this.drives.length; i++) {
            this.scanner.addDrive(this.drives[i]);
          }

          stats.reset();
          this.scanner.execute();
          break;

        case this.stateScanningFiles:
          if (0==this.nTotalHits) {
            this.currentState = this.stateScanComplete;
          } else {
            this.currentState = this.stateScanFound;
          }
          this.bAsyncUpdate = false;
          this.gPrefService.setBoolPref('spywatch.scanning.system', false);
          
          // If we get here without cancelling, update the scan statistics
          if (this.bFullScanComplete) {
            // update the last full scan preference
            var now = new Date;
            this.gPrefService.setIntPref('spywatch.lastFullScan', Math.floor(now.valueOf() / 1000));
            this.gPrefService.setBoolPref('spywatch.initialScan', true);
            this.bFullScanComplete = false;   // reset to initial state
          }
          break;
      }
    }
    
    this.updateDisplay();
    if (this.bAsyncUpdate) {
      this.bAsyncID = setTimeout("systemscan.updateDisplayAsync()", 500);
    }
  },

  startScan : function() {
    nTotalFiles = 0;
    nTotalHits = 0;

    this.gPrefService.setBoolPref('spywatch.scanning.system', true);

    this.scanner.scanDrives = false;
    this.scanner.scanRegistry = false;

    this.scanner.scanMemory = true;
    this.currentState = this.stateScanningMemory;
    this.updateDisplay();
    this.scanner.execute();
          
    // assume we will complete the scan - a user cancel will set this to false
    this.bFullScanComplete = true;

    this.bAsyncUpdate = true;
    this.updateDisplayAsync();
  },

  showOptions : function() {
    window.openDialog('chrome://browser/content/spywarescan/fullscanoptions.xul',
      undefined,
      'modal,centerscreen');

    this.updateDisplay();
  },

  sysClose : function(evt) {
    /* User clicked the Close Window button on the titlebar - need to
     * check whether the scan is running and if it is display the cancel prompt.
     */
     
    if (!this.bAsyncUpdate)
      // no async operations - just close
      return;
    
    if (this.currentState == this.stateRemoving)
    {
      // cannot abort the removal
      evt.preventDefault();
      return;
    }
    else
    {
      // scanning for spyware
      // see if the user wishes to exit the scan
      this.stopScanningPrompt();
      if (this.bFullScanComplete)
      {
        // user chose not to cancel the scan
        evt.preventDefault();
        return;
      }
      
      this.bAsyncShutdown = true;
      this.scanner.cancel = true;
    }

    // kill the async operation that is already in place
    if (this.bAsyncID != 0)
      clearTimeout(this.bAsyncID);

    evt.preventDefault();   // don't close window until we have allowed the scanner or cleaner to cancel
    
    // wait until the scanner component has completed
    setTimeout("systemscan.asyncCloseWindow()", 250);
  },
  
  asyncCloseWindow : function() {
    // wait until the scanner has actually closed
    var stats = this.collection.QueryInterface(Components.interfaces.nsISpywareScanStatistics);
    if (stats.isClosed)
      window.close();
    else
      setTimeout("systemscan.asyncCloseWindow()", 250);
  },

  stopScanningPrompt : function() {
    const params = Components.classes[this.CID_PARAMITEM]
                             .createInstance(Components.interfaces.nsIDialogParamBlock);
    /* pass in params: integer index 0 will be:
     *  0 = continue
     *  1 = cancel scan
     */

    //const ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
    //              .getService(Components.interfaces.nsIWindowWatcher);

    window.openDialog('chrome://browser/content/spywarescan/stopscan.xul',
                      undefined,'modal,centerscreen', params);


    if (1 == params.GetInt(0)) {
      this.scanner.cancel = true;
      this.bFullScanComplete = false;  // indicates that the scan did not complete
    }
  },


  updateRemoveDisplayAsync : function() {
    var percentDone = this.cleaner.percentComplete;
    var isDone = this.cleaner.isDone;

    var bar
    if (this.bAllSpyware) {
      bar = document.getElementById('pm4');
    } else {
      bar = document.getElementById('pm5');
    }
    bar.value = percentDone;

    if (isDone) {
      this.bAllCleaned = this.cleaner.isSuccessful;
      this.bNeedReboot = this.cleaner.isRebootRequired;
    
      if (this.bAllCleaned) {
        this.currentState = this.stateSummary;
        this.nTotalCleaned = this.nTotalHits - this.nIgnoredCount;

      } else {
        this.currentState = this.stateProblems;
        this.nSpywareToClean = 0;
        this.nTotalCleaned = 0;
      }
      this.bAsyncUpdate = false;
    }

    this.updateDisplay();
    if (this.bAsyncUpdate) {
      setTimeout("systemscan.updateRemoveDisplayAsync()", 500);
    }
  },

  applyRDFChange : function(item, bDelete) {
    var svc = Components.classes[this.CID_RDFSVC].createInstance(Components.interfaces.nsIRDFService);
    var listCollection = this.collection.QueryInterface(Components.interfaces.nsISpywareListCollection);
    var ds = listCollection.getDataSource();

    var resPest = svc.GetResource(item);
    var resDelete = svc.GetResource(this.RDF_DELETE);
    var litOldValue, litNewValue;
    
    if (bDelete) {
      var litOldValue = svc.GetLiteral('false');
      var litNewValue = svc.GetLiteral('true');
    } else {
      var litOldValue = svc.GetLiteral('true');
      var litNewValue = svc.GetLiteral('false');
    }
    ds.Change(resPest, resDelete, litOldValue, litNewValue);
  },
  
  applyTableChange : function() {
    var table = document.getElementById('scanlistfound.listbox');
    table.builder.rebuild();
  },
  
  getRemoveCount : function() {  
    var listbox = document.getElementById('scanlistfound.listbox');
    var count = 0;
    if (listbox) {
      var checkbox;
      var item
      var listchildren = listbox.childNodes;
      for (var listindex = 0; listindex < listchildren.length; listindex++) {
        item = listchildren[listindex];

        // iterate through the child nodes of the listitem
        var childcell = item.childNodes[item.childNodes.length-1];  // last column
        var children = childcell.childNodes;
        checkbox = null;
        for (var index = 0; index < children.length; index++) {
          var child = children[index];
          if (child instanceof Components.interfaces.nsIDOMXULCheckboxElement) {
            checkbox = child;
            break;
          }
        }

        if (checkbox) {
          if (checkbox.checked) {
            // add each item that is checked
            count++;
          }
        }
      }
    }
    
    return count;
  },

  remove : function() {
    try {
      if (!this.cleaner) {
        this.cleaner = Components.classes[this.CID_CLEANER].createInstance(Components.interfaces.nsISpywareCleaner);
      }
    } catch (e) {
      throw e;
    }

    this.currentState = this.stateRemoving;
    this.updateDisplay();

    var scanCollection = this.collection.QueryInterface(Components.interfaces.nsISpywareScanCollection);
    this.cleaner.execute(scanCollection);

    this.bAsyncUpdate = true;
    this.updateRemoveDisplayAsync();
  },

  toggleFoundCheck : function(evt) {
    // an item was clicked in the found pests listbox
    
    // don't propagate - we will handle this directly
    evt.currentTarget.selected=false;
    evt.stopPropagation();
    
    var item = evt.target;
    var listbox = item.parentNode;
    var childcell = item.childNodes[item.childNodes.length-1];  // last column

    var checkbox = null;
    var children = childcell.childNodes;
    for (var index = 0; index < children.length; index++) {
      var child = children[index];
      if (child instanceof Components.interfaces.nsIDOMXULCheckboxElement) {
        checkbox = child;
        break;
      }
    }   
    
    // Flip first, then update the ignore list
    this.applyRDFChange(checkbox.getAttribute('rdfid'), !checkbox.checked);
    this.applyTableChange();

    this.updateRemoveButtonsDisplay();

    return false;
  },

  listCheckAll : function(state) {
    var listbox = document.getElementById('scanlistfound.listbox');
    if (listbox) {
      var checkbox;
      var item
      var listchildren = listbox.childNodes;
      for (var listindex = 0; listindex < listchildren.length; listindex++) {
        item = listchildren[listindex];

        // iterate through the child nodes of the listitem
        var childcell = item.childNodes[item.childNodes.length-1];  // last column
        var children = childcell.childNodes;
        checkbox = null;
        for (var index = 0; index < children.length; index++) {
          var child = children[index];
          if (child instanceof Components.interfaces.nsIDOMXULCheckboxElement) {
            checkbox = child;
            break;
          }
        }

        if (checkbox) {
          // set the checkbox state
          this.applyRDFChange(checkbox.getAttribute('rdfid'), state);
        }
      }
      this.applyTableChange();
      this.updateRemoveButtonsDisplay();
    }
  },
  
  fillSummaryList : function(ds, list) {
    // Iterate through the dataset.  For each pest that is not deleted,
    // flag its error msg and give a link to more details
    var Pests;
    var resources = ds.GetAllResources();
    var bFound = false;
    while (resources.hasMoreElements()) {
      Pests = resources.getNext();
      if (Pests instanceof Components.interfaces.nsIRDFResource) {
        if (Pests.EqualsString(this.RDF_PESTS)) {
          bFound = true;
          break;
        }
      }
    }
    
    if (bFound) {
      var container = Components.classes[this.CID_RDFCONTAINER]
                        .createInstance(Components.interfaces.nsIRDFContainer);
      container.Init(ds, Pests);
      
      var utils = Components.classes[this.CID_RDFCONTUTIL]
                    .createInstance(Components.interfaces.nsIRDFContainerUtils);

      var rdfService = Components.classes[this.CID_RDFSVC]
                        .getService(Components.interfaces.nsIRDFService);
                   
      // set up resources
      var resName =  rdfService.GetResource(this.RDF_NAME);
      var resDeleted = rdfService.GetResource(this.RDF_CLEANED);
      var resErrorMsg = rdfService.GetResource(this.RDF_ERRMSG);
      var resID = rdfService.GetResource(this.RDF_ID);
      var resSelected = rdfService.GetResource(this.RDF_DELETE);

      var litName, litDeleted, litMsg, litID, litSelected;
      var bDeleted, bSelected;

      // iterate through each pest
      var children = container.GetElements();
      while (children.hasMoreElements()) {
        var child = children.getNext();
        if (child instanceof Components.interfaces.nsIRDFResource) {
          litSelected = (ds.GetTarget(child, resSelected, true)).QueryInterface(Components.interfaces.nsIRDFLiteral);
          litName = (ds.GetTarget(child, resName, true)).QueryInterface(Components.interfaces.nsIRDFLiteral);
          litDeleted = (ds.GetTarget(child, resDeleted, true)).QueryInterface(Components.interfaces.nsIRDFLiteral);
          try
          {
            // AAM: It seems that occasionally this node does not exist.
            litMsg = (ds.GetTarget(child, resErrorMsg, true)).QueryInterface(Components.interfaces.nsIRDFLiteral);
          }
          catch (e) 
          {
            litMsg = ""
          }
          litID = (ds.GetTarget(child, resID, true)).QueryInterface(Components.interfaces.nsIRDFLiteral);
          
          bDeleted = (litDeleted.Value == 'true');
          bSelected = (litSelected.Value == 'true');
          if (bSelected) {
            this.nSpywareToClean = this.nSpywareToClean + 1;
            if (bDeleted) {
              this.nTotalCleaned = this.nTotalCleaned + 1;
            }
            list.appendChild(this.createSummaryItem(litName.Value, bDeleted, litMsg.Value, litID.Value))
          }          
        }
      }
    }
  },
  
  createSummaryItem : function(Name, bStatus, Msg, ID) {
    var row = document.createElement('listitem');
    row.setAttribute('id', ID);
    row.setAttribute('onmouseup', 'systemscan.showScanHelp(event);');
    
    var cellName = document.createElement('listcell');
    var cellStatus = document.createElement('listcell');
    
    var textStatus;
    var moreInfo;
    cellName.setAttribute('label', Name);
    cellName.setAttribute('crop', 'end');
    
    if (bStatus) {
      textStatus = this.strbundle.getString('summarystatusremoved');    
      cellStatus.setAttribute('label', textStatus + ' ' + Msg);
      cellStatus.setAttribute('class', 'spywareStatusTextOK');
      
    } else {
      textStatus = this.strbundle.getString('summarystatusnotremoved');
      moreInfo = this.strbundle.getString('summarystatusmoreinfo');

      var label = document.createElement('label');
      label.setAttribute('class', 'spywareStatusTextError');
      label.setAttribute('value', textStatus + ' ' + Msg);
      cellStatus.appendChild(label);

      label = document.createElement('label');
      label.setAttribute('class', 'spywareStatusTextLink');
      label.setAttribute('value', moreInfo);
      label.setAttribute('helpID', ID);
      cellStatus.appendChild(label);      
    }
    
    row.appendChild(cellName);
    row.appendChild(cellStatus);
    
    return row;    
  },
  
  showScanHelp : function (evt) {
    var link = evt.target;
    if (link.hasAttribute('id')) {
      var params = Components.classes[this.CID_PARAMITEM].createInstance(Components.interfaces.nsIDialogParamBlock);
      var objArray = Components.classes[this.CID_ARRAY].createInstance(Components.interfaces.nsIMutableArray);
      var id = link.getAttribute('id');

      objArray.insertElementAt(this.collection, 0, false);
      params.objects = objArray;
      params.SetNumberStrings(1);
      params.SetString(0, id);

      window.openDialog('chrome://browser/content/spywarescan/scanhelp.xul',
                        undefined,'modal,centerscreen', params);
    }
  },
  
  displayProblemsDone : function () {
    this.currentState = this.stateSummary;
    this.updateDisplay();
  },
  
  finished : function(bRestart) {
    this.cleanupDatasource();
    
    // called at end of system scan/clean
    if (bRestart && this.bNeedReboot) {
      if (!this.scanner.reboot()) {
        // unable to initiate the reboot - prompt user to reboot manually.
        window.openDialog('chrome://browser/content/spywarescan/rebooterror.xul',
                          undefined,'modal,centerscreen', null);
      }
    }    
    window.close();
  },

  cleanupDatasource : function() {  
  	// BLT Bug #204989 (AAM)
	  // I need to remove the datasource from the listbox prior to shutting down
	  try {
      var table = document.getElementById('scanlistfound.listbox');
      var listCollection = this.collection.QueryInterface(Components.interfaces.nsISpywareListCollection);
      var ds = listCollection.getDataSource();

      table.database.RemoveDataSource(ds);
      table.builder.rebuild();
	  }
	  catch(e) {
	  }
	}
}
